@@ -125,7 +125,6 @@ KoFilter::ConversionStatus Filterkpr2odf::convert(const QByteArray& from, const
     output->write(*preview);
     output->close();
     output->leaveDirectory();
-    manifest->addManifestEntry(QLatin1String("Thumbnails/"), QString());
     manifest->addManifestEntry(QLatin1String("Thumbnails/thumbnail.png"), QString());
     delete preview;
 
@@ -162,12 +161,13 @@ KoFilter::ConversionStatus Filterkpr2odf::convert(const QByteArray& from, const
 
     settings->endElement();//office:document-settings
     settings->endDocument();
+    delete settings;
     output->close();
     manifest->addManifestEntry("settings.xml", "text/xml");
 
     //Create the meta.xml file
     output->open("meta.xml");
-    KoDocumentInfo* meta = new KoDocumentInfo();
+    KoDocumentInfo *meta = new KoDocumentInfo();
     meta->load(m_documentInfo);
     meta->saveOasis(output);
     delete meta;
