@@ -1,6 +1,6 @@
 /* This file is part of the KDE project
    Copyright (C) 2004 Cedric Pasteur <cedric.pasteur@free.fr>
-   Copyright (C) 2004-2009 Jarosław Staniek <staniek@kde.org>
+   Copyright (C) 2004-2012 Jarosław Staniek <staniek@kde.org>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -56,9 +56,7 @@ public:
     KexiFormScrollView(QWidget *parent, bool preview);
     virtual ~KexiFormScrollView();
 
-    void setForm(KFormDesigner::Form *form) {
-        m_form = form;
-    }
+    void setForm(KFormDesigner::Form *form);
 
     /*! Reimplemented from KexiDataAwareObjectInterface
      for checking 'readOnly' flag from a widget
@@ -74,18 +72,11 @@ public:
      Reimplemented for KexiDataAwareObjectInterface:
      column data corresponding to widget number is used here
      (see fieldNumberForColumn()). */
-    virtual KexiTableViewColumn* column(int col);
+    virtual KexiDB::TableViewColumn* column(int col);
 
     /*! \return field number within data model connected to a data-aware
      widget at column \a col. */
-    virtual int fieldNumberForColumn(int col) {
-        KexiFormDataItemInterface *item = dynamic_cast<KexiFormDataItemInterface*>(
-                                              dbFormWidget()->orderedDataAwareWidgets()->at(col));
-        if (!item)
-            return -1;
-        KexiFormDataItemInterfaceToIntMap::ConstIterator it(m_fieldNumbersForDataItems.find(item));
-        return it != m_fieldNumbersForDataItems.constEnd() ? (int)it.value() : -1;
-    }
+    virtual int fieldNumberForColumn(int col);
 
     /*! @internal Used by KexiFormView in view switching. */
     void beforeSwitchView();
@@ -161,7 +152,7 @@ signals:
     void currentItemDeleteRequest();
     void newItemAppendedForAfterDeletingInSpreadSheetMode(); //!< does nothing
     void dataRefreshed();
-    void dataSet(KexiTableViewData *data);
+    void dataSet(KexiDB::TableViewData *data);
     void itemSelected(KexiDB::RecordData*);
     void cellSelected(int col, int row);
     void sortedColumnChanged(int col);
@@ -173,20 +164,20 @@ signals:
 protected slots:
     void slotResizingStarted();
 
-    //! Handles KexiTableViewData::rowRepaintRequested() signal
+    //! Handles KexiDB::TableViewData::rowRepaintRequested() signal
     virtual void slotRowRepaintRequested(KexiDB::RecordData& record);
 
-    //! Handles KexiTableViewData::aboutToDeleteRow() signal. Prepares info for slotRowDeleted().
+    //! Handles KexiDB::TableViewData::aboutToDeleteRow() signal. Prepares info for slotRowDeleted().
     virtual void slotAboutToDeleteRow(KexiDB::RecordData& record, KexiDB::ResultInfo* result, bool repaint) {
         KexiDataAwareObjectInterface::slotAboutToDeleteRow(record, result, repaint);
     }
 
-    //! Handles KexiTableViewData::rowDeleted() signal to repaint when needed.
+    //! Handles KexiDB::TableViewData::rowDeleted() signal to repaint when needed.
     virtual void slotRowDeleted() {
         KexiDataAwareObjectInterface::slotRowDeleted();
     }
 
-    //! Handles KexiTableViewData::rowInserted() signal to repaint when needed.
+    //! Handles KexiDB::TableViewData::rowInserted() signal to repaint when needed.
     virtual void slotRowInserted(KexiDB::RecordData* record, bool repaint);
 
     //! Like above, not db-aware version
@@ -199,7 +190,7 @@ protected slots:
     }
 
     /*! Reloads data for this widget.
-     Handles KexiTableViewData::reloadRequested() signal. */
+     Handles KexiDB::TableViewData::reloadRequested() signal. */
     virtual void reloadData() {
         KexiDataAwareObjectInterface::reloadData();
     }
@@ -282,6 +273,12 @@ protected:
      label should be displayed. */
     virtual bool cursorAtNewRow() const;
 
+    /*! Implementation for KexiFormDataProvider. */
+    virtual void lengthExceeded(KexiDataItemInterface *item, bool lengthExceeded);
+
+    /*! Implementation for KexiFormDataProvider. */
+    virtual void updateLengthExceededMessage(KexiDataItemInterface *item);
+
     //! Implementation for KexiDataAwareObjectInterface
     //! Called by KexiDataAwareObjectInterface::setCursorPosition()
     //! if cursor's position is really changed.
@@ -311,10 +308,9 @@ protected:
 
     //virtual bool focusNextPrevChild( bool next );
 
-    KFormDesigner::Form *m_form;
-    int m_currentLocalSortColumn, m_localSortingOrder;
-    //! Used in selectCellInternal() to avoid fetching the same record twice
-    KexiDB::RecordData *m_previousRecord;
+private:
+    class Private;
+    Private * const d;
 };
 
 #endif
