@@ -40,7 +40,7 @@
 #include "kexidbform.h"
 
 #include <kexi_global.h>
-#include <kexidb/queryschema.h>
+#include <db/queryschema.h>
 #include <formeditor/utils.h>
 #include <kexiutils/utils.h>
 
@@ -95,8 +95,8 @@ KexiDBAutoField::KexiDBAutoField(QWidget *parent, LabelPosition pos)
 KexiDBAutoField::~KexiDBAutoField()
 {
     setUpdatesEnabled(false);
-    if (m_subwidget)
-        m_subwidget->setUpdatesEnabled(false);
+    if (subwidget())
+        subwidget()->setUpdatesEnabled(false);
     delete d;
 }
 
@@ -105,7 +105,7 @@ KexiDBAutoField::init(const QString &text, WidgetType type, LabelPosition pos)
 {
     d->fieldTypeInternal = KexiDB::Field::InvalidType;
     d->layout = 0;
-    m_subwidget = 0;
+    setSubwidget(0);
     d->label = new QLabel(text, this);
     d->label->installEventFilter(this);
     //QFontMetrics fm( font() );
@@ -140,8 +140,8 @@ KexiDBAutoField::setWidgetType(WidgetType type)
 void
 KexiDBAutoField::createEditor()
 {
-    if (m_subwidget) {
-        delete(QWidget *)m_subwidget;
+    if (subwidget()) {
+        delete(QWidget *)subwidget();
     }
 
     QWidget *newSubwidget;
@@ -212,17 +212,17 @@ KexiDBAutoField::createEditor()
 
 void KexiDBAutoField::copyPropertiesToEditor()
 {
-    kDebug() << m_subwidget;
-    if (m_subwidget) {
+    kDebug() << subwidget();
+    if (subwidget()) {
 //  kDebug() << "base col: " <<  d->baseColor.name() <<
 //   "; text col: " << d->textColor.name();
-        QPalette p(m_subwidget->palette());
+        QPalette p(subwidget()->palette());
         p.setBrush(QPalette::Base, d->baseBrush);
         if (d->widgetType == Boolean)
             p.setBrush(QPalette::Foreground, d->textBrush);
         else
             p.setBrush(QPalette::Text, d->textBrush);
-        m_subwidget->setPalette(p);
+        subwidget()->setPalette(p);
         //m_subwidget->setPaletteBackgroundColor( d->baseColor );
     }
 }
@@ -237,8 +237,8 @@ KexiDBAutoField::setLabelPosition(LabelPosition position)
         delete lyr;
     }
 
-    if (m_subwidget)
-        m_subwidget->show();
+    if (subwidget())
+        subwidget()->show();
     //! \todo support right-to-left layout where positions are inverted
     if (position == Top || position == Left) {
         Qt::Alignment align = d->label->alignment();
@@ -263,13 +263,13 @@ KexiDBAutoField::setLabelPosition(LabelPosition position)
         d->layout->addWidget(d->label, 0, position == Top ? Qt::AlignLeft : QFlags<Qt::AlignmentFlag>(0));
         if (position == Left && d->widgetType != Boolean)
             d->layout->addSpacing(KexiDBAutoField_SPACING);
-        d->layout->addWidget(m_subwidget, 1);
-        KexiSubwidgetInterface *subwidgetInterface = dynamic_cast<KexiSubwidgetInterface*>((QWidget*)m_subwidget);
+        d->layout->addWidget(subwidget(), 1);
+        KexiSubwidgetInterface *subwidgetInterface = dynamic_cast<KexiSubwidgetInterface*>((QWidget*)subwidget());
         if (subwidgetInterface) {
             if (subwidgetInterface->appendStretchRequired(this))
                 d->layout->addStretch(0);
             if (subwidgetInterface->subwidgetStretchRequired(this)) {
-                QSizePolicy sizePolicy(m_subwidget->sizePolicy());
+                QSizePolicy sizePolicy(subwidget()->sizePolicy());
                 if (position == Left) {
                     sizePolicy.setHorizontalPolicy(QSizePolicy::Minimum);
                     d->label->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Preferred);
@@ -277,7 +277,7 @@ KexiDBAutoField::setLabelPosition(LabelPosition position)
                     sizePolicy.setVerticalPolicy(QSizePolicy::Minimum);
                     d->label->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Fixed);
                 }
-                m_subwidget->setSizePolicy(sizePolicy);
+                subwidget()->setSizePolicy(sizePolicy);
             }
         }
 //  if(m_subwidget)
@@ -285,14 +285,14 @@ KexiDBAutoField::setLabelPosition(LabelPosition position)
     } else {
         d->layout = (QBoxLayout*) new QHBoxLayout(this);
         d->label->hide();
-        d->layout->addWidget(m_subwidget);
+        d->layout->addWidget(subwidget());
     }
     //a hack to force layout to be refreshed (any better idea for this?)
     resize(size() + QSize(1, 0));
     resize(size() - QSize(1, 0));
-    if (dynamic_cast<KexiDBAutoField*>((QWidget*)m_subwidget)) {
+    if (dynamic_cast<KexiDBAutoField*>((QWidget*)subwidget())) {
         //needed for KexiDBComboBox
-        dynamic_cast<KexiDBAutoField*>((QWidget*)m_subwidget)->setLabelPosition(position);
+        dynamic_cast<KexiDBAutoField*>((QWidget*)subwidget())->setLabelPosition(position);
     }
 }
 
@@ -305,8 +305,8 @@ KexiDBAutoField::setInvalidState(const QString &text)
     d->widgetType = Auto;
     createEditor();
     setFocusPolicy(Qt::NoFocus);
-    if (m_subwidget)
-        m_subwidget->setFocusPolicy(Qt::NoFocus);
+    if (subwidget())
+        subwidget()->setFocusPolicy(Qt::NoFocus);
 //! @todo or set this to editor's text?
     d->label->setText(text);
 }
@@ -314,7 +314,7 @@ KexiDBAutoField::setInvalidState(const QString &text)
 bool
 KexiDBAutoField::isReadOnly() const
 {
-    KexiFormDataItemInterface *iface = dynamic_cast<KexiFormDataItemInterface*>((QWidget*)m_subwidget);
+    KexiFormDataItemInterface *iface = dynamic_cast<KexiFormDataItemInterface*>((QWidget*)subwidget());
     if (iface)
         return iface->isReadOnly();
     else
@@ -324,7 +324,7 @@ KexiDBAutoField::isReadOnly() const
 void
 KexiDBAutoField::setReadOnly(bool readOnly)
 {
-    KexiFormDataItemInterface *iface = dynamic_cast<KexiFormDataItemInterface*>((QWidget*)m_subwidget);
+    KexiFormDataItemInterface *iface = dynamic_cast<KexiFormDataItemInterface*>((QWidget*)subwidget());
     if (iface)
         iface->setReadOnly(readOnly);
 }
@@ -332,16 +332,16 @@ KexiDBAutoField::setReadOnly(bool readOnly)
 void
 KexiDBAutoField::setValueInternal(const QVariant& add, bool removeOld)
 {
-    KexiFormDataItemInterface *iface = dynamic_cast<KexiFormDataItemInterface*>((QWidget*)m_subwidget);
+    KexiFormDataItemInterface *iface = dynamic_cast<KexiFormDataItemInterface*>((QWidget*)subwidget());
     if (iface)
-        iface->setValue(m_origValue, add, removeOld);
+        iface->setValue(KexiDataItemInterface::originalValue(), add, removeOld);
 //  iface->setValueInternal(add, removeOld);
 }
 
 QVariant
 KexiDBAutoField::value()
 {
-    KexiFormDataItemInterface *iface = dynamic_cast<KexiFormDataItemInterface*>((QWidget*)m_subwidget);
+    KexiFormDataItemInterface *iface = dynamic_cast<KexiFormDataItemInterface*>((QWidget*)subwidget());
     if (iface)
         return iface->value();
     return QVariant();
@@ -350,7 +350,7 @@ KexiDBAutoField::value()
 bool
 KexiDBAutoField::valueIsNull()
 {
-    KexiFormDataItemInterface *iface = dynamic_cast<KexiFormDataItemInterface*>((QWidget*)m_subwidget);
+    KexiFormDataItemInterface *iface = dynamic_cast<KexiFormDataItemInterface*>((QWidget*)subwidget());
     if (iface)
         return iface->valueIsNull();
     return true;
@@ -359,7 +359,7 @@ KexiDBAutoField::valueIsNull()
 bool
 KexiDBAutoField::valueIsEmpty()
 {
-    KexiFormDataItemInterface *iface = dynamic_cast<KexiFormDataItemInterface*>((QWidget*)m_subwidget);
+    KexiFormDataItemInterface *iface = dynamic_cast<KexiFormDataItemInterface*>((QWidget*)subwidget());
     if (iface)
         return iface->valueIsEmpty();
     return true;
@@ -368,7 +368,7 @@ KexiDBAutoField::valueIsEmpty()
 bool
 KexiDBAutoField::valueIsValid()
 {
-    KexiFormDataItemInterface *iface = dynamic_cast<KexiFormDataItemInterface*>((QWidget*)m_subwidget);
+    KexiFormDataItemInterface *iface = dynamic_cast<KexiFormDataItemInterface*>((QWidget*)subwidget());
     if (iface)
         return iface->valueIsValid();
     return true;
@@ -377,8 +377,8 @@ KexiDBAutoField::valueIsValid()
 bool
 KexiDBAutoField::valueChanged()
 {
-    KexiFormDataItemInterface *iface = dynamic_cast<KexiFormDataItemInterface*>((QWidget*)m_subwidget);
-    kDebug() << m_origValue;
+    KexiFormDataItemInterface *iface = dynamic_cast<KexiFormDataItemInterface*>((QWidget*)subwidget());
+    kDebug() << KexiDataItemInterface::originalValue();
     if (iface)
         return iface->valueChanged();
     return false;
@@ -388,7 +388,7 @@ void
 KexiDBAutoField::installListener(KexiDataItemChangesListener* listener)
 {
     KexiFormDataItemInterface::installListener(listener);
-    KexiFormDataItemInterface *iface = dynamic_cast<KexiFormDataItemInterface*>((QWidget*)m_subwidget);
+    KexiFormDataItemInterface *iface = dynamic_cast<KexiFormDataItemInterface*>((QWidget*)subwidget());
     if (iface)
         iface->installListener(listener);
 }
@@ -415,7 +415,7 @@ bool KexiDBAutoField::hasAutoCaption() const
 
 QWidget* KexiDBAutoField::editor() const
 {
-    return m_subwidget;
+    return subwidget();
 }
 
 QLabel* KexiDBAutoField::label() const
@@ -436,7 +436,7 @@ QString KexiDBAutoField::fieldCaptionInternal() const
 bool
 KexiDBAutoField::cursorAtStart()
 {
-    KexiFormDataItemInterface *iface = dynamic_cast<KexiFormDataItemInterface*>((QWidget*)m_subwidget);
+    KexiFormDataItemInterface *iface = dynamic_cast<KexiFormDataItemInterface*>((QWidget*)subwidget());
     if (iface)
         return iface->cursorAtStart();
     return false;
@@ -445,7 +445,7 @@ KexiDBAutoField::cursorAtStart()
 bool
 KexiDBAutoField::cursorAtEnd()
 {
-    KexiFormDataItemInterface *iface = dynamic_cast<KexiFormDataItemInterface*>((QWidget*)m_subwidget);
+    KexiFormDataItemInterface *iface = dynamic_cast<KexiFormDataItemInterface*>((QWidget*)subwidget());
     if (iface)
         return iface->cursorAtEnd();
     return false;
@@ -454,7 +454,7 @@ KexiDBAutoField::cursorAtEnd()
 void
 KexiDBAutoField::clear()
 {
-    KexiFormDataItemInterface *iface = dynamic_cast<KexiFormDataItemInterface*>((QWidget*)m_subwidget);
+    KexiFormDataItemInterface *iface = dynamic_cast<KexiFormDataItemInterface*>((QWidget*)subwidget());
     if (iface)
         iface->clear();
 }
@@ -487,7 +487,7 @@ KexiDBAutoField::setFieldCaptionInternal(const QString& text)
 {
     d->fieldCaptionInternal = text;
     //change text only if autocaption is set and no columnInfo is available
-    KexiFormDataItemInterface *iface = dynamic_cast<KexiFormDataItemInterface*>((QWidget*)m_subwidget);
+    KexiFormDataItemInterface *iface = dynamic_cast<KexiFormDataItemInterface*>((QWidget*)subwidget());
     if ((!iface || !iface->columnInfo()) && d->autoCaption) {
         changeText(d->fieldCaptionInternal);
     }
@@ -525,7 +525,7 @@ KexiDBAutoField::setColumnInfoInternal(KexiDB::QueryColumnInfo* cinfo, KexiDB::Q
     // update label's text
     changeText((cinfo && d->autoCaption) ? cinfo->captionOrAliasOrName() : d->caption);
 
-    KexiFormDataItemInterface *iface = dynamic_cast<KexiFormDataItemInterface*>((QWidget*)m_subwidget);
+    KexiFormDataItemInterface *iface = dynamic_cast<KexiFormDataItemInterface*>((QWidget*)subwidget());
     if (iface)
         iface->setColumnInfo(visibleColumnInfo);
 }
@@ -602,7 +602,7 @@ KexiDBAutoField::changeText(const QString &text, bool beautify)
     }
 // QWidget* widgetToAlterForegroundColor;
     if (d->widgetType == Boolean) {
-        static_cast<QCheckBox*>((QWidget*)m_subwidget)->setText(realText);
+        static_cast<QCheckBox*>((QWidget*)subwidget())->setText(realText);
 //  widgetToAlterForegroundColor = m_subwidget;
     } else {
         d->label->setText(realText);
@@ -654,11 +654,11 @@ QSize
 KexiDBAutoField::sizeHint() const
 {
     if (d->lblPosition == NoLabel)
-        return m_subwidget ? m_subwidget->sizeHint() : QWidget::sizeHint();
+        return subwidget() ? subwidget()->sizeHint() : QWidget::sizeHint();
 
     QSize s1(0, 0);
-    if (m_subwidget)
-        s1 = m_subwidget->sizeHint();
+    if (subwidget())
+        s1 = subwidget()->sizeHint();
     QSize s2(d->label->sizeHint());
     if (d->lblPosition == Top)
         return QSize(qMax(s1.width(), s2.width()), s1.height() + KexiDBAutoField_SPACING + s2.height());
@@ -673,8 +673,8 @@ KexiDBAutoField::setFocusPolicy(Qt::FocusPolicy policy)
     d->focusPolicyChanged = true;
     QWidget::setFocusPolicy(policy);
     d->label->setFocusPolicy(policy);
-    if (m_subwidget)
-        m_subwidget->setFocusPolicy(policy);
+    if (subwidget())
+        subwidget()->setFocusPolicy(policy);
 }
 
 void
@@ -826,34 +826,34 @@ bool KexiDBAutoField::eventFilter(QObject *o, QEvent *e)
 void KexiDBAutoField::setDisplayDefaultValue(QWidget* widget, bool displayDefaultValue)
 {
     KexiFormDataItemInterface::setDisplayDefaultValue(widget, displayDefaultValue);
-    if (dynamic_cast<KexiFormDataItemInterface*>((QWidget*)m_subwidget))
-        dynamic_cast<KexiFormDataItemInterface*>((QWidget*)m_subwidget)->setDisplayDefaultValue(m_subwidget, displayDefaultValue);
+    if (dynamic_cast<KexiFormDataItemInterface*>((QWidget*)subwidget()))
+        dynamic_cast<KexiFormDataItemInterface*>((QWidget*)subwidget())->setDisplayDefaultValue(subwidget(), displayDefaultValue);
 }
 
 void KexiDBAutoField::moveCursorToEnd()
 {
-    KexiDataItemInterface *iface = dynamic_cast<KexiDataItemInterface*>((QWidget*)m_subwidget);
+    KexiDataItemInterface *iface = dynamic_cast<KexiDataItemInterface*>((QWidget*)subwidget());
     if (iface)
         iface->moveCursorToEnd();
 }
 
 void KexiDBAutoField::moveCursorToStart()
 {
-    KexiDataItemInterface *iface = dynamic_cast<KexiDataItemInterface*>((QWidget*)m_subwidget);
+    KexiDataItemInterface *iface = dynamic_cast<KexiDataItemInterface*>((QWidget*)subwidget());
     if (iface)
         iface->moveCursorToStart();
 }
 
 void KexiDBAutoField::selectAll()
 {
-    KexiDataItemInterface *iface = dynamic_cast<KexiDataItemInterface*>((QWidget*)m_subwidget);
+    KexiDataItemInterface *iface = dynamic_cast<KexiDataItemInterface*>((QWidget*)subwidget());
     if (iface)
         iface->selectAll();
 }
 
 bool KexiDBAutoField::keyPressed(QKeyEvent *ke)
 {
-    KexiFormDataItemInterface *iface = dynamic_cast<KexiFormDataItemInterface*>((QWidget*)m_subwidget);
+    KexiFormDataItemInterface *iface = dynamic_cast<KexiFormDataItemInterface*>((QWidget*)subwidget());
     if (iface && iface->keyPressed(ke))
         return true;
     return false;
