@@ -34,11 +34,12 @@
 
 
 PresetDockerDock::PresetDockerDock( )
-    : QDockWidget(i18n("Preset docker"))
+    : QDockWidget(i18n("Brush Presets"))
     , m_canvas(0)
 {
     m_presetChooser = new KisPaintOpPresetsChooserPopup(this);
     m_presetChooser->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
+    m_presetChooser->showButtons(false);
     setWidget(m_presetChooser);
 }
 
