@@ -40,9 +40,10 @@
 #include <kis_color_source.h>
 #include <kis_pressure_sharpness_option.h>
 #include <KoColorSpaceRegistry.h>
+#include <kis_fixed_paint_device.h>
 
 KisBrushOp::KisBrushOp(const KisBrushBasedPaintOpSettings *settings, KisPainter *painter, KisImageWSP image)
-        : KisBrushBasedPaintOp(settings, painter), m_hsvTransfo(0)
+    : KisBrushBasedPaintOp(settings, painter), m_opacityOption(settings->node()), m_hsvTransformation(0)
 {
     Q_UNUSED(image);
     Q_ASSERT(settings);
@@ -59,9 +60,9 @@ KisBrushOp::KisBrushOp(const KisBrushBasedPaintOpSettings *settings, KisPainter
     {
         option->readOptionSetting(settings);
         option->sensor()->reset();
-        if(option->isChecked() && !m_hsvTransfo)
+        if(option->isChecked() && !m_hsvTransformation)
         {
-            m_hsvTransfo = painter->backgroundColor().colorSpace()->createColorTransformation("hsv_adjustment", QHash<QString, QVariant>());
+            m_hsvTransformation = painter->backgroundColor().colorSpace()->createColorTransformation("hsv_adjustment", QHash<QString, QVariant>());
         }
     }
 
@@ -84,13 +85,16 @@ KisBrushOp::KisBrushOp(const KisBrushBasedPaintOpSettings *settings, KisPainter
     m_darkenOption.sensor()->reset();
     m_rotationOption.sensor()->reset();
     m_scatterOption.sensor()->reset();
+
+    m_dabCache->setMirrorPostprocessing(&m_mirrorOption);
+    m_dabCache->setSharpnessPostprocessing(&m_sharpnessOption);
 }
 
 KisBrushOp::~KisBrushOp()
 {
     qDeleteAll(m_hsvOptions);
     delete m_colorSource;
-    delete m_hsvTransfo;
+    delete m_hsvTransformation;
 }
 
 qreal KisBrushOp::paintAt(const KisPaintInformation& info)
@@ -115,7 +119,7 @@ qreal KisBrushOp::paintAt(const KisPaintInformation& info)
     setCurrentScale(scale);
     setCurrentRotation(rotation);
 
-    QPointF hotSpot = brush->hotSpot(scale, scale, rotation);
+    QPointF hotSpot = brush->hotSpot(scale, scale, rotation, info);
     // return info.pos() if sensor is not enabled
     QPointF pos = m_scatterOption.apply(info, qMax(brush->width(), brush->height()) * scale);
     QPointF pt = pos - hotSpot;
@@ -132,53 +136,37 @@ qreal KisBrushOp::paintAt(const KisPaintInformation& info)
 
     quint8 origOpacity = painter()->opacity();
     quint8 origFlow    = painter()->flow();
-    
+
     m_opacityOption.apply(painter(), info);
     m_colorSource->selectColor(m_mixOption.apply(info));
     m_darkenOption.apply(m_colorSource, info);
 
-    if(m_hsvTransfo)
-    {
-        foreach(KisPressureHSVOption* option, m_hsvOptions)
-        {
-            option->apply(m_hsvTransfo, info);
+    if (m_hsvTransformation) {
+        foreach(KisPressureHSVOption* option, m_hsvOptions) {
+            option->apply(m_hsvTransformation, info);
         }
-        m_colorSource->applyColorTransformation(m_hsvTransfo);
+        m_colorSource->applyColorTransformation(m_hsvTransformation);
     }
 
-    KisFixedPaintDeviceSP dab = cachedDab(device->colorSpace());
-    if (brush->brushType() == IMAGE || brush->brushType() == PIPE_IMAGE) {
-        dab = brush->paintDevice(device->colorSpace(), scale, rotation, info, xFraction, yFraction);
-    } else {
-        if(m_colorSource->isUniformColor())
-        {
-            KoColor color = m_colorSource->uniformColor();
-            color.convertTo(dab->colorSpace());
-            brush->mask(dab, color, scale, scale, rotation, info, xFraction, yFraction, m_softnessOption.apply(info));
-        } else {
-            if (!m_colorSourceDevice) {
-                m_colorSourceDevice = new KisPaintDevice(dab->colorSpace());
-            } else {
-                m_colorSourceDevice->clear();
-            }
-            m_colorSource->colorize(m_colorSourceDevice, QRect(0, 0, brush->maskWidth(scale, rotation), brush->maskHeight(scale, rotation)), info.pos().toPoint() );
-            brush->mask(dab, m_colorSourceDevice, scale, scale, rotation, info, xFraction, yFraction, m_softnessOption.apply(info));
-        }
-    }
-
-    MirrorProperties mirrors = m_mirrorOption.apply(info);
-    dab->mirror(mirrors.horizontalMirror, mirrors.verticalMirror);
-
-    m_sharpnessOption.applyTreshold( dab );
+    KisFixedPaintDeviceSP dab = m_dabCache->fetchDab(device->compositionSourceColorSpace(),
+                                                     m_colorSource,
+                                                     scale, scale,
+                                                     rotation,
+                                                     info,
+                                                     xFraction, yFraction,
+                                                     m_softnessOption.apply(info));
 
     painter()->bltFixed(QPoint(x, y), dab, dab->bounds());
-    painter()->renderMirrorMask(QRect(QPoint(x,y), QSize(dab->bounds().width(),dab->bounds().height())),dab);
+
+    painter()->renderMirrorMaskSafe(QRect(QPoint(x,y), QSize(dab->bounds().width(),dab->bounds().height())),
+                                    dab,
+                                    !m_dabCache->needSeparateOriginal());
     painter()->setOpacity(origOpacity);
     painter()->setFlow(origFlow);
 
-    if(m_spacingOption.isChecked())
+    if (m_spacingOption.isChecked())
         return spacing(m_spacingOption.apply(info));
-    
+
     return spacing(scale);
 }
 
@@ -186,18 +174,18 @@ KisDistanceInformation KisBrushOp::paintLine(const KisPaintInformation& pi1, con
 {
     if(m_sharpnessOption.isChecked() && m_brush && (m_brush->width() == 1) && (m_brush->height() == 1)) {
 
-        if (!m_dab) {
-            m_dab = new KisPaintDevice(painter()->device()->colorSpace());
+        if (!m_lineCacheDevice) {
+            m_lineCacheDevice = source()->createCompositionSourceDevice();
         } else {
-            m_dab->clear();
+            m_lineCacheDevice->clear();
         }
 
-        KisPainter p(m_dab);
+        KisPainter p(m_lineCacheDevice);
         p.setPaintColor(painter()->paintColor());
         p.drawDDALine(pi1.pos(), pi2.pos());
 
-        QRect rc = m_dab->extent();
-        painter()->bitBlt(rc.x(), rc.y(), m_dab, rc.x(), rc.y(), rc.width(), rc.height());
+        QRect rc = m_lineCacheDevice->extent();
+        painter()->bitBlt(rc.x(), rc.y(), m_lineCacheDevice, rc.x(), rc.y(), rc.width(), rc.height());
 
         return KisDistanceInformation(0.0, 0.0);
     }
