@@ -32,6 +32,8 @@
 #include "kis_paint_device.h"
 #include "kis_paint_layer.h"
 
+#include "testutil.h"
+
 
 KisImageSP utils::createImage(KisUndoStore *undoStore, const QSize &imageSize) {
     QRect imageRect(0,0,imageSize.width(),imageSize.height());
@@ -89,14 +91,15 @@ KoCanvasResourceManager* utils::createResourceManager(KisImageWSP image,
 
     KisPaintOpPresetSP preset;
 
-    if(!presetFileName.isEmpty()) {
-        QString dataPath = QString(FILES_DATA_DIR) + QDir::separator();
-        preset = new KisPaintOpPreset(dataPath + presetFileName);
-        preset->load();
-    }
+    if (!presetFileName.isEmpty()) {
+        QString fullFileName = TestUtil::fetchDataFileLazy(presetFileName);
+        preset = new KisPaintOpPreset(fullFileName);
+        bool presetValid = preset->load();
+        Q_ASSERT(presetValid);
 
-    i.setValue(preset);
-    manager->setResource(KisCanvasResourceProvider::CurrentPaintOpPreset, i);
+        i.setValue(preset);
+        manager->setResource(KisCanvasResourceProvider::CurrentPaintOpPreset, i);
+    }
 
     i.setValue(COMPOSITE_OVER);
     manager->setResource(KisCanvasResourceProvider::CurrentCompositeOp, i);
@@ -178,11 +181,13 @@ void utils::StrokeTester::testOneStroke(bool cancelled,
     QImage refImage;
     refImage.load(referenceFile(testName));
 
-    if(resultImage != refImage) {
+    QPoint temp;
+    if(!TestUtil::compareQImages(temp, refImage, resultImage, 1, 1)) {
+        refImage.save(dumpReferenceFile(testName));
         resultImage.save(resultFile(testName));
-    }
 
-    QCOMPARE(resultImage, refImage);
+        QFAIL("Images do not coincide");
+    }
 }
 
 QString utils::StrokeTester::formatTestName(const QString &baseName,
@@ -209,6 +214,15 @@ QString utils::StrokeTester::referenceFile(const QString &testName)
     return path;
 }
 
+QString utils::StrokeTester::dumpReferenceFile(const QString &testName)
+{
+    QString path = QString(FILES_OUTPUT_DIR) + QDir::separator();
+    path += testName;
+    path += "_expected";
+    path += ".png";
+    return path;
+}
+
 QString utils::StrokeTester::resultFile(const QString &testName)
 {
     QString path = QString(FILES_OUTPUT_DIR) + QDir::separator();
