@@ -4,7 +4,7 @@
  * Copyright (C) 2006 Peter Simonsson <peter.simonsson@gmail.com>
  * Copyright (C) 2006, 2009 Thorsten Zachmann <zachmann@kde.org>
  * Copyright (C) 2007-2010 Boudewijn Rempt <boud@valdyas.org>
- * Copyright (C) 2007 Casper Boemann <cbr@boemann.dk>
+ * Copyright (C) 2007 C. Boemann <cbo@boemann.dk>
  * Copyright (C) 2006-2008 Jan Hambrecht <jaham@gmx.net>
  *
  * This library is free software; you can redistribute it and/or
@@ -24,6 +24,7 @@
  */
 
 #include "KoCanvasControllerWidget.h"
+#include "KoCanvasControllerWidget_p.h"
 
 #include "KoCanvasControllerWidgetViewport_p.h"
 #include "KoShape.h"
@@ -34,7 +35,7 @@
 #include "KoToolManager_p.h"
 
 #include <ksharedconfig.h>
-#include <KDebug>
+#include <kdebug.h>
 #include <kconfiggroup.h>
 #include <QApplication>
 #include <QMouseEvent>
@@ -47,45 +48,10 @@
 #include <KoConfig.h>
 
 #ifdef HAVE_OPENGL
-#include <QtOpenGL/QGLWidget>
+#include <QGLWidget>
 #endif
 
 
-class KoCanvasControllerWidget::Private
-{
-public:
-
-    Private(KoCanvasControllerWidget *qq)
-        : q(qq)
-        , canvas(0)
-        , lastActivatedCanvas(0)
-        , ignoreScrollSignals(false)
-        , zoomWithWheel(false)
-        , vastScrollingFactor(0)
-    {
-    }
-
-    /**
-     * Gets called by the tool manager if this canvas controller is the current active canvas controller.
-     */
-    void setDocumentOffset();
-
-    void resetScrollBars();
-    void emitPointerPositionChangedSignals(QEvent *event);
-
-    void activate();
-    void unsetCanvas();
-
-    KoCanvasControllerWidget *q;
-    KoCanvasBase *canvas;
-    KoCanvasBase *lastActivatedCanvas;
-    Viewport *viewportWidget;
-    bool ignoreScrollSignals;
-    bool zoomWithWheel;
-    qreal vastScrollingFactor;
-};
-
-
 void KoCanvasControllerWidget::Private::setDocumentOffset()
 {
     // The margins scroll the canvas widget inside the viewport, not
@@ -197,7 +163,7 @@ void KoCanvasControllerWidget::Private::activate()
         foreach(KoCanvasObserverBase *docker, observerProvider->canvasObservers()) {
             KoCanvasObserverBase *observer = dynamic_cast<KoCanvasObserverBase*>(docker);
             if (observer) {
-                observer->setCanvas(q->canvas());
+                observer->setObservedCanvas(q->canvas());
             }
         }
         lastActivatedCanvas = q->canvas();
@@ -216,8 +182,8 @@ void KoCanvasControllerWidget::Private::unsetCanvas()
     }
     foreach(KoCanvasObserverBase *docker, observerProvider->canvasObservers()) {
         KoCanvasObserverBase *observer = dynamic_cast<KoCanvasObserverBase*>(docker);
-        if (observer) {
-            observer->unsetCanvas();
+        if (observer && observer->observedCanvas() == q->canvas()) {
+            observer->unsetObservedCanvas();
         }
     }
 }
@@ -358,14 +324,10 @@ int KoCanvasControllerWidget::visibleWidth() const
 
 int KoCanvasControllerWidget::canvasOffsetX() const
 {
-    int offset = 0;
+    int offset = -horizontalScrollBar()->value();
 
     if (d->canvas) {
-        offset = d->canvas->canvasWidget()->x() + frameWidth();
-    }
-
-    if (horizontalScrollBar()) {
-        offset -= horizontalScrollBar()->value();
+        offset += d->canvas->canvasWidget()->x() + frameWidth();
     }
 
     return offset;
@@ -373,14 +335,10 @@ int KoCanvasControllerWidget::canvasOffsetX() const
 
 int KoCanvasControllerWidget::canvasOffsetY() const
 {
-    int offset = 0;
+    int offset = -verticalScrollBar()->value();
 
     if (d->canvas) {
-        offset = d->canvas->canvasWidget()->y() + frameWidth();
-    }
-
-    if (verticalScrollBar()) {
-        offset -= verticalScrollBar()->value();
+        offset += d->canvas->canvasWidget()->y() + frameWidth();
     }
 
     return offset;
@@ -391,11 +349,9 @@ void KoCanvasControllerWidget::updateCanvasOffsetX()
     proxyObject->emitCanvasOffsetXChanged(canvasOffsetX());
     if (d->ignoreScrollSignals)
         return;
-    if (horizontalScrollBar()->isVisible())
-        setPreferredCenterFractionX((horizontalScrollBar()->value()
-                                     + horizontalScrollBar()->pageStep() / 2.0) / documentSize().width());
-    else
-        setPreferredCenterFractionX(0);
+
+    setPreferredCenterFractionX((horizontalScrollBar()->value()
+                                 + viewport()->width() / 2.0) / documentSize().width());
 }
 
 void KoCanvasControllerWidget::updateCanvasOffsetY()
@@ -403,11 +359,9 @@ void KoCanvasControllerWidget::updateCanvasOffsetY()
     proxyObject->emitCanvasOffsetYChanged(canvasOffsetY());
     if (d->ignoreScrollSignals)
         return;
-    if (verticalScrollBar()->isVisible())
-        setPreferredCenterFractionY((verticalScrollBar()->value()
-                                     + verticalScrollBar()->pageStep() / 2.0) / documentSize().height());
-    else
-        setPreferredCenterFractionY(0);
+
+    setPreferredCenterFractionY((verticalScrollBar()->value()
+                                 + viewport()->height() / 2.0) / documentSize().height());
 }
 
 bool KoCanvasControllerWidget::eventFilter(QObject *watched, QEvent *event)
@@ -467,25 +421,19 @@ void KoCanvasControllerWidget::recenterPreferred()
     const bool oldIgnoreScrollSignals = d->ignoreScrollSignals;
     d->ignoreScrollSignals = true;
 
-    QPoint center = QPoint(int(documentSize().width() * preferredCenterFractionX()),
-                           int(documentSize().height() * preferredCenterFractionY()));
+    QPointF center = preferredCenter();
 
     // convert into a viewport based point
     center.rx() += d->canvas->canvasWidget()->x() + frameWidth();
     center.ry() += d->canvas->canvasWidget()->y() + frameWidth();
 
-    scrollToCenterPoint(center);
+    // scroll to a new center point
+    QPointF topLeft = center - 0.5 * QPointF(viewport()->width(), viewport()->height());
+    setScrollBarValue(topLeft.toPoint());
 
     d->ignoreScrollSignals = oldIgnoreScrollSignals;
 }
 
-void KoCanvasControllerWidget::scrollToCenterPoint(const QPoint &center)
-{
-    // calculate the difference to the viewport centerpoint
-    QPoint topLeft = center - 0.5 * QPoint(viewport()->width(), viewport()->height());
-    setScrollBarValue(topLeft);
-}
-
 void KoCanvasControllerWidget::zoomIn(const QPoint &center)
 {
     zoomBy(center, sqrt(2.0));
@@ -503,10 +451,8 @@ void KoCanvasControllerWidget::zoomBy(const QPoint &center, qreal zoom)
 
     const bool oldIgnoreScrollSignals = d->ignoreScrollSignals;
     d->ignoreScrollSignals = true;
-    proxyObject->emitZoomBy(zoom);
+    proxyObject->emitZoomRelative(zoom, preferredCenter());
     d->ignoreScrollSignals = oldIgnoreScrollSignals;
-    recenterPreferred();
-    d->canvas->canvasWidget()->update();
 }
 
 void KoCanvasControllerWidget::zoomTo(const QRect &viewRect)
@@ -518,15 +464,7 @@ void KoCanvasControllerWidget::zoomTo(const QRect &viewRect)
     else
         scale = 1.0 * viewport()->width() / viewRect.width();
 
-    const qreal preferredCenterFractionX = 1.0 * viewRect.center().x() / documentSize().width();
-    const qreal preferredCenterFractionY = 1.0 * viewRect.center().y() / documentSize().height();
-
-    proxyObject->emitZoomBy(scale);
-
-    setPreferredCenterFractionX(preferredCenterFractionX);
-    setPreferredCenterFractionY(preferredCenterFractionY);
-    recenterPreferred();
-    d->canvas->canvasWidget()->update();
+    zoomBy(viewRect.center(), scale);
 }
 
 void KoCanvasControllerWidget::setToolOptionWidgets(const QList<QWidget *>&widgetMap)
@@ -575,18 +513,18 @@ void KoCanvasControllerWidget::pan(const QPoint &distance)
     setScrollBarValue(sourcePoint + distance);
 }
 
-void KoCanvasControllerWidget::setPreferredCenter(const QPoint &viewPoint)
+void KoCanvasControllerWidget::setPreferredCenter(const QPointF &viewPoint)
 {
-    setPreferredCenterFractionX(1.0 * viewPoint.x() / documentSize().width());
-    setPreferredCenterFractionY(1.0 * viewPoint.y() / documentSize().height());
+    setPreferredCenterFractionX(viewPoint.x() / documentSize().width());
+    setPreferredCenterFractionY(viewPoint.y() / documentSize().height());
     recenterPreferred();
 }
 
-QPoint KoCanvasControllerWidget::preferredCenter() const
+QPointF KoCanvasControllerWidget::preferredCenter() const
 {
-    QPoint center;
-    center.setX(qRound(preferredCenterFractionX() * documentSize().width()));
-    center.setY(qRound(preferredCenterFractionY() * documentSize().height()));
+    QPointF center;
+    center.setX(preferredCenterFractionX() * documentSize().width());
+    center.setY(preferredCenterFractionY() * documentSize().height());
     return center;
 }
 
@@ -624,28 +562,23 @@ void KoCanvasControllerWidget::keyPressEvent(QKeyEvent *event)
 void KoCanvasControllerWidget::wheelEvent(QWheelEvent *event)
 {
     if (d->zoomWithWheel != ((event->modifiers() & Qt::ControlModifier) == Qt::ControlModifier)) {
-        const bool oldIgnoreScrollSignals = d->ignoreScrollSignals;
-        d->ignoreScrollSignals = true;
-
-        const qreal zoomLevel = event->delta() > 0 ? sqrt(2.0) : sqrt(0.5);
-        zoomRelativeToPoint(event->pos(), zoomLevel);
+        const qreal zoomCoeff = event->delta() > 0 ? sqrt(2.0) : sqrt(0.5);
+        zoomRelativeToPoint(event->pos(), zoomCoeff);
 
         event->accept();
-
-        d->ignoreScrollSignals = oldIgnoreScrollSignals;
     } else
         QAbstractScrollArea::wheelEvent(event);
 }
 
-void KoCanvasControllerWidget::zoomRelativeToPoint(const QPoint &widgetPoint, qreal zoomLevel)
+void KoCanvasControllerWidget::zoomRelativeToPoint(const QPoint &widgetPoint, qreal zoomCoeff)
 {
-    const QPoint offset(horizontalScrollBar()->value(), verticalScrollBar()->value());
+    const QPoint offset = scrollBarValue();
     const QPoint mousePos(widgetPoint + offset);
 
-    QPointF oldCenter = preferredCenter();
-    const QPointF newCenter = mousePos - (1.0 / zoomLevel) * (mousePos - oldCenter);
-
-    zoomBy(newCenter.toPoint(), zoomLevel);
+    const bool oldIgnoreScrollSignals = d->ignoreScrollSignals;
+    d->ignoreScrollSignals = true;
+    proxyObject->emitZoomRelative(zoomCoeff, mousePos);
+    d->ignoreScrollSignals = oldIgnoreScrollSignals;
 }
 
 bool KoCanvasControllerWidget::focusNextPrevChild(bool)
@@ -665,27 +598,17 @@ QPoint KoCanvasControllerWidget::scrollBarValue() const
 {
     QScrollBar * hBar = horizontalScrollBar();
     QScrollBar * vBar = verticalScrollBar();
-    QPoint value;
-    if (hBar && !hBar->isHidden()) {
-        value.setX(hBar->value());
-    }
-    if (vBar && !vBar->isHidden()) {
-        value.setY(vBar->value());
-    }
 
-    return value;
+    return QPoint(hBar->value(), vBar->value());
 }
 
 void KoCanvasControllerWidget::setScrollBarValue(const QPoint &value)
 {
     QScrollBar * hBar = horizontalScrollBar();
     QScrollBar * vBar = verticalScrollBar();
-    if (hBar && !hBar->isHidden()) {
-        hBar->setValue(value.x());
-    }
-    if (vBar && !vBar->isHidden()) {
-        vBar->setValue(value.y());
-    }
+
+    hBar->setValue(value.x());
+    vBar->setValue(value.y());
 }
 
 KoCanvasControllerWidget::Private *KoCanvasControllerWidget::priv()
