@@ -25,13 +25,13 @@
 
 #include "changetracker/KoChangeTracker.h"
 #include "changetracker/KoChangeTrackerElement.h"
-#include "changetracker/KoDeleteChangeMarker.h"
 #include "styles/KoCharacterStyle.h"
 #include "styles/KoParagraphStyle.h"
 #include "styles/KoStyleManager.h"
 #include "commands/TextPasteCommand.h"
+#include "commands/ParagraphFormattingCommand.h"
 
-#include <KLocale>
+#include <klocale.h>
 #include <kundo2stack.h>
 
 #include <QApplication>
@@ -77,12 +77,8 @@ void KoTextEditor::bold(bool bold)
     d->updateState(KoTextEditor::Private::Format, i18nc("(qtundo-format)", "Bold"));
     QTextCharFormat format;
     format.setFontWeight(bold ? QFont::Bold : QFont::Normal);
-
-    QTextCharFormat prevFormat(d->caret.charFormat());
-    d->caret.mergeCharFormat(format);
-    registerTrackedChange(d->caret, KoGenChange::FormatChange, i18nc("(qtundo-format)", "Bold"), format, prevFormat, false);
+    mergeAutoStyle(format);
     d->updateState(KoTextEditor::Private::NoOp);
-    emit textFormatChanged();
 }
 
 void KoTextEditor::italic(bool italic)
@@ -94,12 +90,8 @@ void KoTextEditor::italic(bool italic)
     d->updateState(KoTextEditor::Private::Format, i18nc("(qtundo-format)", "Italic"));
     QTextCharFormat format;
     format.setFontItalic(italic);
-
-    QTextCharFormat prevFormat(d->caret.charFormat());
-    d->caret.mergeCharFormat(format);
-    registerTrackedChange(d->caret, KoGenChange::FormatChange, i18nc("(qtundo-format)", "Italic"), format, prevFormat, false);
+    mergeAutoStyle(format);
     d->updateState(KoTextEditor::Private::NoOp);
-    emit textFormatChanged();
 }
 
 void KoTextEditor::underline(bool underline)
@@ -117,12 +109,8 @@ void KoTextEditor::underline(bool underline)
         format.setProperty(KoCharacterStyle::UnderlineType, KoCharacterStyle::NoLineType);
         format.setProperty(KoCharacterStyle::UnderlineStyle, KoCharacterStyle::NoLineStyle);
     }
-
-    QTextCharFormat prevFormat(d->caret.charFormat());
-    d->caret.mergeCharFormat(format);
-    registerTrackedChange(d->caret, KoGenChange::FormatChange, i18nc("(qtundo-format)", "Underline"), format, prevFormat, false);
+    mergeAutoStyle(format);
     d->updateState(KoTextEditor::Private::NoOp);
-    emit textFormatChanged();
 }
 
 void KoTextEditor::strikeOut(bool strikeout)
@@ -140,11 +128,8 @@ void KoTextEditor::strikeOut(bool strikeout)
         format.setProperty(KoCharacterStyle::StrikeOutType, KoCharacterStyle::NoLineType);
         format.setProperty(KoCharacterStyle::StrikeOutStyle, KoCharacterStyle::NoLineStyle);
     }
-    QTextCharFormat prevFormat(d->caret.charFormat());
-    d->caret.mergeCharFormat(format);
-    registerTrackedChange(d->caret, KoGenChange::FormatChange, i18nc("(qtundo-format)", "Strike Out"), format, prevFormat, false);
+    mergeAutoStyle(format);
     d->updateState(KoTextEditor::Private::NoOp);
-    emit textFormatChanged();
 }
 
 void KoTextEditor::setHorizontalTextAlignment(Qt::Alignment align)
@@ -185,14 +170,11 @@ void KoTextEditor::setVerticalTextAlignment(Qt::Alignment align)
     else if (align == Qt::AlignBottom)
         charAlign = QTextCharFormat::AlignSubScript;
 
-    d->updateState(KoTextEditor::Private::Format, i18n("Set Vertical Alignment"));
+    d->updateState(KoTextEditor::Private::Format, i18nc("(qtundo-format)", "Set Vertical Alignment"));
     QTextCharFormat format;
     format.setVerticalAlignment(charAlign);
-    QTextCharFormat prevFormat(d->caret.charFormat());
-    d->caret.mergeCharFormat(format);
-    registerTrackedChange(d->caret, KoGenChange::FormatChange, i18n("Set Vertical Alignment"), format, prevFormat, false);
+    mergeAutoStyle(format);
     d->updateState(KoTextEditor::Private::NoOp);
-    emit textFormatChanged();
 }
 
 void KoTextEditor::decreaseIndent()
@@ -222,8 +204,8 @@ void KoTextEditor::decreaseIndent()
     };
 
     Indenter indenter;
-    d->updateState(KoTextEditor::Private::Format, i18n("Decrease Indent"));
-    BlockFormatVisitor::visitSelection(this, indenter, i18n("Decrease Indent"));
+    d->updateState(KoTextEditor::Private::Format, i18nc("(qtundo-format)", "Decrease Indent"));
+    BlockFormatVisitor::visitSelection(this, indenter, i18nc("(qtundo-format)", "Decrease Indent"));
     d->updateState(KoTextEditor::Private::NoOp);
     emit textFormatChanged();
 }
@@ -258,8 +240,8 @@ void KoTextEditor::increaseIndent()
     };
 
     Indenter indenter;
-    d->updateState(KoTextEditor::Private::Format, i18n("Increase Indent"));
-    BlockFormatVisitor::visitSelection(this, indenter, i18n("Increase Indent"));
+    d->updateState(KoTextEditor::Private::Format, i18nc("(qtundo-format)", "Increase Indent"));
+    BlockFormatVisitor::visitSelection(this, indenter, i18nc("(qtundo-format)", "Increase Indent"));
     d->updateState(KoTextEditor::Private::NoOp);
     emit textFormatChanged();
 }
@@ -294,9 +276,9 @@ void KoTextEditor::decreaseFontSize()
         return;
     }
 
-    d->updateState(KoTextEditor::Private::Format, i18n("Decrease font size"));
+    d->updateState(KoTextEditor::Private::Format, i18nc("(qtundo-format)", "Decrease font size"));
     FontResizer sizer(FontResizer::Shrink);
-    CharFormatVisitor::visitSelection(this, sizer, i18n("Decrease font size"));
+    CharFormatVisitor::visitSelection(this, sizer, i18nc("(qtundo-format)", "Decrease font size"));
     d->updateState(KoTextEditor::Private::NoOp);
     emit textFormatChanged();
 }
@@ -307,9 +289,9 @@ void KoTextEditor::increaseFontSize()
         return;
     }
 
-    d->updateState(KoTextEditor::Private::Format, i18n("Increase font size"));
+    d->updateState(KoTextEditor::Private::Format, i18nc("(qtundo-format)", "Increase font size"));
     FontResizer sizer(FontResizer::Grow);
-    CharFormatVisitor::visitSelection(this, sizer, i18n("Increase font size"));
+    CharFormatVisitor::visitSelection(this, sizer, i18nc("(qtundo-format)", "Increase font size"));
     d->updateState(KoTextEditor::Private::NoOp);
     emit textFormatChanged();
 }
@@ -320,14 +302,11 @@ void KoTextEditor::setFontFamily(const QString &font)
         return;
     }
 
-    d->updateState(KoTextEditor::Private::Format, i18n("Set Font"));
+    d->updateState(KoTextEditor::Private::Format, i18nc("(qtundo-format)", "Set Font"));
     QTextCharFormat format;
     format.setFontFamily(font);
-    QTextCharFormat prevFormat(d->caret.charFormat());
-    d->caret.mergeCharFormat(format);
-    registerTrackedChange(d->caret, KoGenChange::FormatChange, i18n("Set Font"), format, prevFormat, false);
+    mergeAutoStyle(format);
     d->updateState(KoTextEditor::Private::NoOp);
-    emit textFormatChanged();
 }
 
 void KoTextEditor::setFontSize(qreal size)
@@ -336,14 +315,11 @@ void KoTextEditor::setFontSize(qreal size)
         return;
     }
 
-    d->updateState(KoTextEditor::Private::Format, i18n("Set Font Size"));
+    d->updateState(KoTextEditor::Private::Format, i18nc("(qtundo-format)", "Set Font Size"));
     QTextCharFormat format;
     format.setFontPointSize(size);
-    QTextCharFormat prevFormat(d->caret.charFormat());
-    d->caret.mergeCharFormat(format);
-    registerTrackedChange(d->caret, KoGenChange::FormatChange, i18n("Set Font Size"), format, prevFormat, false);
+    mergeAutoStyle(format);
     d->updateState(KoTextEditor::Private::NoOp);
-    emit textFormatChanged();
 }
 
 void KoTextEditor::setTextBackgroundColor(const QColor &color)
@@ -352,14 +328,11 @@ void KoTextEditor::setTextBackgroundColor(const QColor &color)
         return;
     }
 
-    d->updateState(KoTextEditor::Private::Format, i18n("Set Background Color"));
+    d->updateState(KoTextEditor::Private::Format, i18nc("(qtundo-format)", "Set Background Color"));
     QTextCharFormat format;
     format.setBackground(QBrush(color));
-    QTextCharFormat prevFormat(d->caret.charFormat());
-    d->caret.mergeCharFormat(format);
-    registerTrackedChange(d->caret, KoGenChange::FormatChange, i18n("Set Background Color"), format, prevFormat, false);
+    mergeAutoStyle(format);
     d->updateState(KoTextEditor::Private::NoOp);
-    emit textFormatChanged();
 }
 
 void KoTextEditor::setTextColor(const QColor &color)
@@ -368,14 +341,11 @@ void KoTextEditor::setTextColor(const QColor &color)
         return;
     }
 
-    d->updateState(KoTextEditor::Private::Format, i18n("Set Text Color"));
+    d->updateState(KoTextEditor::Private::Format, i18nc("(qtundo-format)", "Set Text Color"));
     QTextCharFormat format;
     format.setForeground(QBrush(color));
-    QTextCharFormat prevFormat(d->caret.charFormat());
-    d->caret.mergeCharFormat(format);
-    registerTrackedChange(d->caret, KoGenChange::FormatChange, i18n("Set Text Color"), format, prevFormat, false);
+    mergeAutoStyle(format);
     d->updateState(KoTextEditor::Private::NoOp);
-    emit textFormatChanged();
 }
 
 class SetCharacterStyleVisitor : public KoTextVisitor
@@ -387,7 +357,7 @@ public:
     {
     }
 
-    virtual void visitBlock(QTextBlock block, const QTextCursor &caret)
+    virtual void visitBlock(QTextBlock &block, const QTextCursor &caret)
     {
         m_newFormat = block.charFormat();
         m_style->applyStyle(m_newFormat);
@@ -399,12 +369,12 @@ public:
         foreach(QTextCursor cursor, m_cursors) {
             QTextFormat prevFormat(cursor.charFormat());
             cursor.setCharFormat(*it);
-            editor()->registerTrackedChange(cursor, KoGenChange::FormatChange, i18n("Set Character Style"), *it, prevFormat, false);
+            editor()->registerTrackedChange(cursor, KoGenChange::FormatChange, i18nc("(qtundo-format)", "Set Character Style"), *it, prevFormat, false);
             ++it;
         }
     }
 
-    virtual void visitFragmentSelection(QTextCursor fragmentSelection)
+    virtual void visitFragmentSelection(QTextCursor &fragmentSelection)
     {
         QTextCharFormat format = m_newFormat;
 
@@ -421,6 +391,7 @@ public:
 
         if (fragmentSelection.charFormat().isAnchor()) {
             format.setAnchor(true);
+            format.setProperty(KoCharacterStyle::AnchorType, fragmentSelection.charFormat().intProperty(KoCharacterStyle::AnchorType));
             format.setAnchorHref(fragmentSelection.charFormat().anchorHref());
         }
 
@@ -437,13 +408,34 @@ public:
 void KoTextEditor::setStyle(KoCharacterStyle *style)
 {
     Q_ASSERT(style);
-    d->updateState(KoTextEditor::Private::Custom, i18n("Set Character Style"));
+    d->updateState(KoTextEditor::Private::Custom, i18nc("(qtundo-format)", "Set Character Style"));
+
+    int caretAnchor = d->caret.anchor();
+    int caretPosition = d->caret.position();
 
     SetCharacterStyleVisitor visitor(this, style);
 
     recursivelyVisitSelection(d->document->rootFrame()->begin(), visitor);
+
+    if (!isEditProtected() && caretAnchor == caretPosition) { //if there is no selection, it can happen that the caret does not get the proper style applied (begining of a block). We need to force it.
+         //applying a style is absolute, so first initialise the caret with the frame's style, then apply the paragraph's. Finally apply the character style
+        QTextCharFormat charFormat = KoTextDocument(d->document).frameCharFormat();
+        KoStyleManager *styleManager = KoTextDocument(d->document).styleManager();
+        KoParagraphStyle *paragraphStyle = styleManager->paragraphStyle(d->caret.charFormat().intProperty(KoParagraphStyle::StyleId));
+        if (paragraphStyle) {
+            paragraphStyle->KoCharacterStyle::applyStyle(charFormat);
+        }
+        d->caret.setCharFormat(charFormat);
+        style->applyStyle(&(d->caret));
+    }
+    else { //if the caret has a selection, the visitor has already applied the style, reset the caret's position so it picks the proper style.
+        d->caret.setPosition(caretAnchor);
+        d->caret.setPosition(caretPosition, QTextCursor::KeepAnchor);
+    }
+
     d->updateState(KoTextEditor::Private::NoOp);
     emit textFormatChanged();
+    emit characterStyleApplied(style);
 }
 
 
@@ -457,7 +449,7 @@ public:
     {
     }
 
-    virtual void visitBlock(QTextBlock block, const QTextCursor &)
+    virtual void visitBlock(QTextBlock &block, const QTextCursor &)
     {
         if (m_styleManager) {
             QTextBlockFormat bf = block.blockFormat();
@@ -477,14 +469,28 @@ public:
 
 void KoTextEditor::setStyle(KoParagraphStyle *style)
 {
-    d->updateState(KoTextEditor::Private::Custom, i18n("Set Paragraph Style"));
+    d->updateState(KoTextEditor::Private::Custom, i18nc("(qtundo-format)", "Set Paragraph Style"));
 
+    int caretAnchor = d->caret.anchor();
+    int caretPosition = d->caret.position();
     KoStyleManager *styleManager = KoTextDocument(d->document).styleManager();
     SetParagraphStyleVisitor visitor(this, styleManager, style);
 
     recursivelyVisitSelection(d->document->rootFrame()->begin(), visitor);
 
+    if (!isEditProtected() && caretAnchor == caretPosition) { //if there is no selection, it can happen that the caret does not get the proper style applied (begining of a block). We need to force it.
+        //applying a style is absolute, so first initialise the caret with the frame's style, then apply the paragraph style
+        QTextCharFormat charFormat = KoTextDocument(d->document).frameCharFormat();
+        d->caret.setCharFormat(charFormat);
+        style->KoCharacterStyle::applyStyle(&(d->caret));
+    }
+    else {
+        d->caret.setPosition(caretAnchor);
+        d->caret.setPosition(caretPosition, QTextCursor::KeepAnchor);
+    }
+
     d->updateState(KoTextEditor::Private::NoOp);
+    emit paragraphStyleApplied(style);
     emit textFormatChanged();
 }
 
@@ -497,7 +503,7 @@ public:
     {
     }
 
-    virtual void visitBlock(QTextBlock block, const QTextCursor &caret)
+    virtual void visitBlock(QTextBlock &block, const QTextCursor &caret)
     {
         KoTextVisitor::visitBlock(block, caret);
 
@@ -505,12 +511,12 @@ public:
         foreach(QTextCursor cursor, m_cursors) {
             QTextFormat prevFormat(cursor.charFormat());
             cursor.setCharFormat(*it);
-            editor()->registerTrackedChange(cursor, KoGenChange::FormatChange, i18n("Formatting"), *it, prevFormat, false);
+            editor()->registerTrackedChange(cursor, KoGenChange::FormatChange, i18nc("(qtundo-format)", "Formatting"), *it, prevFormat, false);
             ++it;
         }
     }
 
-    virtual void visitFragmentSelection(QTextCursor fragmentSelection)
+    virtual void visitFragmentSelection(QTextCursor &fragmentSelection)
     {
         QTextCharFormat format = fragmentSelection.charFormat();
         format.merge(m_deltaCharFormat);
@@ -524,78 +530,34 @@ public:
     QList<QTextCursor> m_cursors;
 };
 
-void KoTextEditor::mergeAutoStyle(QTextCharFormat deltaCharFormat)
+void KoTextEditor::mergeAutoStyle(const QTextCharFormat &deltaCharFormat)
 {
     d->updateState(KoTextEditor::Private::Custom, "Formatting");
 
+    int caretAnchor = d->caret.anchor();
+    int caretPosition = d->caret.position();
     MergeAutoCharacterStyleVisitor visitor(this, deltaCharFormat);
 
     recursivelyVisitSelection(d->document->rootFrame()->begin(), visitor);
 
+    if (!isEditProtected() && caretAnchor == caretPosition) { //if there is no selection, it can happen that the caret does not get the proper style applied (begining of a block). We need to force it.
+        d->caret.mergeCharFormat(deltaCharFormat);
+    }
+    else {
+        d->caret.setPosition(caretAnchor);
+        d->caret.setPosition(caretPosition, QTextCursor::KeepAnchor);
+    }
+
     d->updateState(KoTextEditor::Private::NoOp);
     emit textFormatChanged();
 }
 
-class MergeAutoParagraphStyleVisitor : public KoTextVisitor
+
+void KoTextEditor::applyDirectFormatting(const QTextCharFormat &deltaCharFormat,
+                                  const QTextBlockFormat &deltaBlockFormat,
+                                  const KoListLevelProperties &llp)
 {
-public:
-    MergeAutoParagraphStyleVisitor(KoTextEditor *editor, QTextCharFormat deltaCharFormat, QTextBlockFormat deltaBlockFormat)
-        : KoTextVisitor(editor)
-        , m_deltaCharFormat(deltaCharFormat)
-        , m_deltaBlockFormat(deltaBlockFormat)
-    {
-    }
-
-    virtual void visitBlock(QTextBlock block, const QTextCursor &caret)
-    {
-        for (QTextBlock::iterator it = block.begin(); it != block.end(); ++it) {
-            QTextCursor fragmentSelection(caret);
-            fragmentSelection.setPosition(it.fragment().position());
-            fragmentSelection.setPosition(it.fragment().position() + it.fragment().length(), QTextCursor::KeepAnchor);
-
-            if (fragmentSelection.anchor() >= fragmentSelection.position()) {
-                continue;
-            }
-
-            visitFragmentSelection(fragmentSelection);
-        }
-
-        QList<QTextCharFormat>::Iterator it = m_formats.begin();
-        foreach(QTextCursor cursor, m_cursors) {
-            QTextFormat prevFormat(cursor.charFormat());
-            cursor.setCharFormat(*it);
-            editor()->registerTrackedChange(cursor, KoGenChange::FormatChange, i18n("Formatting"), *it, prevFormat, false);
-            ++it;
-        }
-        QTextCursor cursor(caret);
-        cursor.mergeBlockFormat(m_deltaBlockFormat);
-        cursor.mergeBlockCharFormat(m_deltaCharFormat);
-    }
-
-    virtual void visitFragmentSelection(QTextCursor fragmentSelection)
-    {
-        QTextCharFormat format = fragmentSelection.charFormat();
-        format.merge(m_deltaCharFormat);
-
-        m_formats.append(format);
-        m_cursors.append(fragmentSelection);
-    }
-
-    QTextCharFormat m_deltaCharFormat;
-    QTextBlockFormat m_deltaBlockFormat;
-    QList<QTextCharFormat> m_formats;
-    QList<QTextCursor> m_cursors;
-};
-
-void KoTextEditor::mergeAutoStyle(QTextCharFormat deltaCharFormat, QTextBlockFormat deltaBlockFormat)
-{
-    d->updateState(KoTextEditor::Private::Custom, "Formatting");
-
-    MergeAutoParagraphStyleVisitor visitor(this, deltaCharFormat, deltaBlockFormat);
-
-    recursivelyVisitSelection(d->document->rootFrame()->begin(), visitor);
-
-    d->updateState(KoTextEditor::Private::NoOp);
+    addCommand(new ParagraphFormattingCommand(this, deltaCharFormat, deltaBlockFormat, llp));
     emit textFormatChanged();
 }
 
