@@ -33,12 +33,12 @@ Boston, MA 02110-1301, USA.
 #include <QApplication>
 #include <QByteArray>
 
-#include <KLocale>
-#include <KMessageBox>
-#include <KLibLoader>
-#include <KSqueezedTextLabel>
-#include <KMimeType>
-#include <KDebug>
+#include <klocale.h>
+#include <kmessagebox.h>
+#include <klibloader.h>
+#include <ksqueezedtextlabel.h>
+#include <kmimetype.h>
+#include <kdebug.h>
 
 #include <queue>
 
@@ -108,7 +108,7 @@ QString KoFilterManager::importDocument(const QString& url,
                     m_document->extraNativeMimeTypes(KoDocument::ForImport)), nativeFormat, u);
             if (chooser.exec()) {
                 QByteArray f = chooser.filterSelected().toLatin1();
-
+                qDebug() << "User choose format" << f;
                 if (f == nativeFormat) {
                     status = KoFilter::OK;
                     QApplication::restoreOverrideCursor();
@@ -227,7 +227,7 @@ KoFilter::ConversionStatus KoFilterManager::exportDocument(const QString& url, Q
 
     if (!m_graph.isValid()) {
         kError(30500) << "Couldn't create a valid graph for this source mimetype.";
-        if (!userCancelled) KMessageBox::error(0, i18n("Could not export file."), i18n("Missing Export Filter"));
+        if (!d->batch && !userCancelled) KMessageBox::error(0, i18n("Could not export file."), i18n("Missing Export Filter"));
         return KoFilter::BadConversionGraph;
     }
 
@@ -236,7 +236,7 @@ KoFilter::ConversionStatus KoFilterManager::exportDocument(const QString& url, Q
 
     if (!chain) {
         kError(30500) << "Couldn't create a valid filter chain to " << mimeType << " !" << endl;
-        KMessageBox::error(0, i18n("Could not export file."), i18n("Missing Export Filter"));
+        if (!d->batch) KMessageBox::error(0, i18n("Could not export file."), i18n("Missing Export Filter"));
         return KoFilter::BadConversionGraph;
     }
 
