@@ -43,6 +43,7 @@
 #include <ksystemtimezone.h>
 #include <ktimezone.h>
 #include <krandom.h>
+#include <kdeversion.h>
 
 
 namespace KPlato
@@ -199,7 +200,7 @@ void Project::calculate( const DateTime &dt )
         cs->notScheduled = false;
         calcFreeFloat();
         emit scheduleChanged( cs );
-        emit changed();
+        emit projectChanged();
     } else if ( type() == Type_Subproject ) {
         kWarning() << "Subprojects not implemented";
     } else {
@@ -240,7 +241,7 @@ void Project::calculate( ScheduleManager &sm )
     emit sigCalculationFinished( this, &sm );
     emit scheduleManagerChanged( &sm );
     emit projectCalculated( &sm );
-    emit changed();
+    emit projectChanged();
     sm.setScheduling( false );
 }
 
@@ -351,7 +352,7 @@ void Project::calculate()
         cs->notScheduled = false;
         calcFreeFloat();
         emit scheduleChanged( cs );
-        emit changed();
+        emit projectChanged();
     } else if ( type() == Type_Subproject ) {
         kWarning() << "Subprojects not implemented";
     } else {
@@ -367,7 +368,7 @@ void Project::finishCalculation( ScheduleManager &sm )
     cs->notScheduled = false;
     calcFreeFloat();
     emit scheduleChanged( cs );
-    emit changed();
+    emit projectChanged();
 }
 
 void Project::setProgress( int progress, ScheduleManager *sm )
@@ -542,6 +543,83 @@ DateTime Project::checkEndConstraints( const DateTime &dt ) const
     return t;
 }
 
+#ifndef PLAN_NLOGDEBUG
+bool Project::checkParent( Node *n, QList<Node*> list, QList<Relation*> &checked )
+{
+    if ( n->isStartNode() ) {
+        kDebug(planDbg())<<n<<"start node"<<list;
+        return true;
+    }
+    kDebug(planDbg())<<"Check:"<<n<<":"<<checked.count()<<":"<<list;
+    if ( list.contains( n ) ) {
+        kDebug(planDbg())<<"Failed:"<<n<<":"<<list;
+        return false;
+    }
+    QList<Node*> lst = list;
+    lst << n;
+    foreach ( Relation *r, n->dependParentNodes() ) {
+        if ( checked.contains( r ) ) {
+            kDebug(planDbg())<<"Depend:"<<n<<":"<<r->parent()<<": checked";
+            continue;
+        }
+        checked << r;
+        if ( ! checkParent( r->parent(), lst, checked ) ) {
+            return false;
+        }
+    }
+    Task *t = static_cast<Task*>( n );
+    foreach ( Relation *r, t->parentProxyRelations() ) {
+        if ( checked.contains( r ) ) {
+            kDebug(planDbg())<<"Depend:"<<n<<":"<<r->parent()<<": checked";
+            continue;
+        }
+        checked << r;
+        kDebug(planDbg())<<"Proxy:"<<n<<":"<<r->parent()<<":"<<lst;
+        if ( ! checkParent( r->parent(), lst, checked ) ) {
+            return false;
+        }
+    }
+    return true;
+}
+
+bool Project::checkChildren( Node *n, QList<Node*> list, QList<Relation*> &checked )
+{
+    if ( n->isEndNode() ) {
+        kDebug(planDbg())<<n<<"end node"<<list;
+        return true;
+    }
+    kDebug(planDbg())<<"Check:"<<n<<":"<<checked.count()<<":"<<list;
+    if ( list.contains( n ) ) {
+        kDebug(planDbg())<<"Failed:"<<n<<":"<<list;
+        return false;
+    }
+    QList<Node*> lst = list;
+    lst << n;
+    foreach ( Relation *r, n->dependChildNodes() ) {
+        if ( checked.contains( r ) ) {
+            kDebug(planDbg())<<"Depend:"<<n<<":"<<r->parent()<<": checked";
+            continue;
+        }
+        checked << r;
+        if ( ! checkChildren( r->child(), lst, checked ) ) {
+            return false;
+        }
+    }
+    Task *t = static_cast<Task*>( n );
+    foreach ( Relation *r, t->childProxyRelations() ) {
+        if ( checked.contains( r ) ) {
+            kDebug(planDbg())<<"Depend:"<<n<<":"<<r->parent()<<": checked";
+            continue;
+        }
+        kDebug(planDbg())<<"Proxy:"<<n<<":"<<r->parent()<<":"<<lst;
+        checked << r;
+        if ( ! checkChildren( r->child(), lst, checked ) ) {
+            return false;
+        }
+    }
+    return true;
+}
+#endif
 void Project::tasksForward()
 {
     m_hardConstraints.clear();
@@ -565,6 +643,14 @@ void Project::tasksForward()
                 break;
         }
     }
+#ifndef PLAN_NLOGDEBUG
+    kDebug(planDbg())<<"End nodes:"<<m_terminalNodes;
+    foreach ( Node* n, m_terminalNodes ) {
+        QList<Node*> lst;
+        QList<Relation*> rel;
+        Q_ASSERT( checkParent( n, lst, rel ) ); Q_UNUSED( n );
+    }
+#endif
 }
 
 void Project::tasksBackward()
@@ -584,12 +670,20 @@ void Project::tasksBackward()
                 m_softConstraints.append( t );
                 break;
             default:
-                if ( t->isEndNode() ) {
+                if ( t->isStartNode() ) {
                     m_terminalNodes.append( t );
                 }
                 break;
         }
     }
+#ifndef PLAN_NLOGDEBUG
+    kDebug(planDbg())<<"Start nodes:"<<m_terminalNodes;
+    foreach ( Node* n, m_terminalNodes ) {
+        QList<Node*> lst;
+        QList<Relation*> rel;
+        Q_ASSERT( checkChildren( n, lst, rel ) ); Q_UNUSED( n );
+    }
+#endif
 }
 
 DateTime Project::calculateForward( int use )
@@ -601,6 +695,9 @@ DateTime Project::calculateForward( int use )
         return finish;
     }
     if ( type() == Node::Type_Project ) {
+        QTime timer;
+        timer.start();
+        cs->logInfo( i18n( "Start calculating forward" ) );
         m_visitedForward = true;
         if ( ! m_visitedBackward ) {
             // setup tasks
@@ -646,6 +743,7 @@ DateTime Project::calculateForward( int use )
                 }
             }
         }
+        cs->logInfo( i18n( "Finished calculating forward: %1 ms", timer.elapsed() ) );
     } else {
         //TODO: subproject
     }
@@ -661,6 +759,9 @@ DateTime Project::calculateBackward( int use )
         return start;
     }
     if ( type() == Node::Type_Project ) {
+        QTime timer;
+        timer.start();
+        cs->logInfo( i18n( "Start calculating backward" ) );
         m_visitedBackward = true;
         if ( ! m_visitedForward ) {
             // setup tasks
@@ -706,6 +807,7 @@ DateTime Project::calculateBackward( int use )
                 }
             }
         }
+        cs->logInfo( i18n( "Finished calculating backward: %1 ms", timer.elapsed() ) );
     } else {
         //TODO: subproject
     }
@@ -719,6 +821,9 @@ DateTime Project::scheduleForward( const DateTime &earliest, int use )
     if ( cs == 0 || stopcalculation ) {
         return DateTime();
     }
+    QTime timer;
+    timer.start();
+    cs->logInfo( i18n( "Start scheduling forward" ) );
     resetVisited();
     // Schedule in the same order as calculated forward
     // Do all hard constrained first
@@ -738,6 +843,13 @@ DateTime Project::scheduleForward( const DateTime &earliest, int use )
     }
     // Fix summarytasks
     adjustSummarytask();
+    cs->logInfo( i18n( "Finished scheduling forward: %1 ms", timer.elapsed() ) );
+    foreach ( Node *n, allNodes() ) {
+        if ( n->type() == Node::Type_Task || n->type() == Node::Type_Milestone ) {
+            Q_ASSERT( n->isScheduled() );
+        }
+    }
+
     return end;
 }
 
@@ -748,6 +860,9 @@ DateTime Project::scheduleBackward( const DateTime &latest, int use )
     if ( cs == 0 || stopcalculation ) {
         return start;
     }
+    QTime timer;
+    timer.start();
+    cs->logInfo( i18n( "Start scheduling backward" ) );
     resetVisited();
     // Schedule in the same order as calculated backward
     // Do all hard constrained first
@@ -767,6 +882,12 @@ DateTime Project::scheduleBackward( const DateTime &latest, int use )
     }
     // Fix summarytasks
     adjustSummarytask();
+    cs->logInfo( i18n( "Finished scheduling backward: %1 ms", timer.elapsed() ) );
+    foreach ( Node *n, allNodes() ) {
+        if ( n->type() == Node::Type_Task || n->type() == Node::Type_Milestone ) {
+            Q_ASSERT( n->isScheduled() );
+        }
+    }
     return start;
 }
 
@@ -827,7 +948,7 @@ bool Project::load( KoXmlElement &element, XMLLoaderObject &status )
 
 //            l->setMonetaryThousandsSeparator( e.attribute( "monetary-thousands-separator", l->monetaryThousandsSeparator() ) );
             if ( e.hasAttribute( "currency-digits" ) ) {
-                l->setFracDigits( e.attribute( "currency-digits" ).toInt() );
+                l->setMonetaryDecimalPlaces(e.attribute("currency-digits").toInt());
             }
             if ( e.hasAttribute( "positive-monetary-sign-position" ) ) {
                 l->setPositiveMonetarySignPosition( (KLocale::SignPosition)( e.attribute( "positive-monetary-sign-position" ).toInt() ) );
@@ -1142,7 +1263,7 @@ void Project::save( QDomElement &element ) const
     loc.setAttribute( "currency-symbol", l->currencySymbol() );
 //    loc.setAttribute( "monetary-decimal-symbol", l->monetaryDecimalSymbol() );
 //    loc.setAttribute( "monetary-thousands-separator", l->monetaryThousandsSeparator() );
-    loc.setAttribute( "currency-digits", l->fracDigits() );
+    loc.setAttribute("currency-digits", l->monetaryDecimalPlaces());
     loc.setAttribute( "positive-monetary-sign-position", l->positiveMonetarySignPosition() );
     loc.setAttribute( "positive-prefix-currency-symbol", l->positivePrefixCurrencySymbol() );
     loc.setAttribute( "negative-monetary-sign-position", l->negativeMonetarySignPosition() );
@@ -1228,11 +1349,13 @@ void Project::saveWorkPackageXML( QDomElement &element, const Node *node, long i
     }
     node->saveWorkPackageXML( me, id );
 
-    ScheduleManager *sm = scheduleManager( id );
-    if ( sm ) {
-        QDomElement el = me.ownerDocument().createElement( "schedules" );
-        me.appendChild( el );
-        sm->saveWorkPackageXML( el, *node );
+    foreach ( ScheduleManager *sm, m_managerIdMap ) {
+        if ( sm->scheduleId() == id ) {
+            QDomElement el = me.ownerDocument().createElement( "schedules" );
+            me.appendChild( el );
+            sm->saveWorkPackageXML( el, *node );
+            break;
+        }
     }
 }
 
@@ -1256,7 +1379,7 @@ void Project::addResourceGroup( ResourceGroup *group, int index )
         r->setProject( this );
     }
     emit resourceGroupAdded( group );
-    emit changed();
+    emit projectChanged();
 }
 
 ResourceGroup *Project::takeResourceGroup( ResourceGroup *group )
@@ -1276,7 +1399,7 @@ ResourceGroup *Project::takeResourceGroup( ResourceGroup *group )
         removeResourceId( r->id() );
     }
     emit resourceGroupRemoved( g );
-    emit changed();
+    emit projectChanged();
     return g;
 }
 
@@ -1292,7 +1415,7 @@ void Project::addResource( ResourceGroup *group, Resource *resource, int index )
     group->addResource( i, resource, 0 );
     setResourceId( resource );
     emit resourceAdded( resource );
-    emit changed();
+    emit projectChanged();
 }
 
 Resource *Project::takeResource( ResourceGroup *group, Resource *resource )
@@ -1310,7 +1433,7 @@ Resource *Project::takeResource( ResourceGroup *group, Resource *resource )
         kWarning() << "Cound not take resource from group";
     }
     emit resourceRemoved( resource );
-    emit changed();
+    emit projectChanged();
     return r;
 }
 
@@ -1391,7 +1514,7 @@ bool Project::addSubTask( Node* task, int index, Node* parent, bool emitSignal )
     connect( this, SIGNAL( standardWorktimeChanged( StandardWorktime* ) ), task, SLOT( slotStandardWorktimeChanged( StandardWorktime* ) ) );
     if ( emitSignal ) {
         emit nodeAdded( task );
-        emit changed();
+        emit projectChanged();
         if ( p != this && p->numChildren() == 1 ) {
             emit nodeChanged( p );
         }
@@ -1413,7 +1536,7 @@ void Project::takeTask( Node *node, bool emitSignal )
     parent->takeChildNode( node );
     if ( emitSignal ) {
         emit nodeRemoved( node );
-        emit changed();
+        emit projectChanged();
         if ( parent != this && parent->type() != Node::Type_Summarytask ) {
             emit nodeChanged( parent );
         }
@@ -1443,10 +1566,15 @@ bool Project::moveTask( Node* node, Node *newParent, int newPos )
         return false;
     }
     Node *oldParent = node->parentNode();
-    const Node *before = newParent->childNode( newPos );
-    emit nodeToBeMoved( node );
+    int oldPos = oldParent->indexOf( node );
+    int i = newPos < 0 ? newParent->numChildren() : newPos;
+    int newRow = i;
+    if ( oldParent == newParent && newPos > oldPos ) {
+        ++newRow; // itemmodels wants new row *before* node is removed from old position
+    }
+    kDebug(planDbg())<<node->name()<<"at"<<oldParent->indexOf( node )<<"to"<<newParent->name()<<i<<newRow<<"("<<newPos<<")";
+    emit nodeToBeMoved( node, oldPos, newParent, newRow );
     takeTask( node, false );
-    int i = before == 0 ? newParent->numChildren() : newPos;
     addSubTask( node, i, newParent, false );
     emit nodeMoved( node );
     if ( oldParent != this && oldParent->numChildren() == 0 ) {
@@ -1767,6 +1895,17 @@ ResourceGroup *Project::groupByName( const QString& name ) const
     return 0;
 }
 
+QList<Resource*> Project::autoAllocateResources() const
+{
+    QList<Resource*> lst;
+    foreach ( Resource *r, resourceIdDict ) {
+        if ( r->autoAllocate() ) {
+            lst << r;
+        }
+    }
+    return lst;
+}
+
 void Project::insertResourceId( const QString &id, Resource *resource )
 {
     resourceIdDict.insert( id, resource );
@@ -1838,6 +1977,10 @@ QStringList Project::resourceNameList() const
 EffortCostMap Project::plannedEffortCostPrDay( const QDate & start, const QDate &end, long id, EffortCostCalculationType typ ) const
 {
     //kDebug(planDbg())<<start<<end<<id;
+    Schedule *s = schedule( id );
+    if ( s == 0 ) {
+        return EffortCostMap();
+    }
     EffortCostMap ec;
     QListIterator<Node*> it( childNodeIterator() );
     while ( it.hasNext() ) {
@@ -2067,7 +2210,7 @@ void Project::addCalendar( Calendar *calendar, Calendar *parent, int index )
     }
     setCalendarId( calendar );
     emit calendarAdded( calendar );
-    emit changed();
+    emit projectChanged();
 }
 
 void Project::takeCalendar( Calendar *calendar )
@@ -2087,7 +2230,7 @@ void Project::takeCalendar( Calendar *calendar )
     }
     emit calendarRemoved( calendar );
     calendar->setProject( 0 );
-    emit changed();
+    emit projectChanged();
 }
 
 int Project::indexOf( const Calendar *calendar ) const
@@ -2171,7 +2314,7 @@ void Project::setDefaultCalendar( Calendar *cal )
         cal->setDefault( true );
     }
     emit defaultCalendarChanged( cal );
-    emit changed();
+    emit projectChanged();
 }
 
 void Project::setStandardWorktime( StandardWorktime * worktime )
@@ -2286,7 +2429,7 @@ void Project::setWbsDefinition( const WBSDefinition &def )
     //kDebug(planDbg());
     m_wbsDefinition = def;
     emit wbsDefinitionChanged();
-    emit changed();
+    emit projectChanged();
 }
 
 QString Project::generateWBSCode( QList<int> &indexes ) const
@@ -2318,7 +2461,7 @@ void Project::setCurrentSchedule( long id )
         r->setCurrentSchedule( id );
     }
     emit currentScheduleChanged();
-    emit changed();
+    emit projectChanged();
 }
 
 ScheduleManager *Project::scheduleManager( long id ) const
@@ -2397,7 +2540,7 @@ void Project::addScheduleManager( ScheduleManager *sm, ScheduleManager *parent,
     m_managerIdMap.insert( sm->managerId(), sm );
 
     emit scheduleManagerAdded( sm );
-    emit changed();
+    emit projectChanged();
     //kDebug(planDbg())<<"Added:"<<sm->name()<<", now"<<m_managers.count();
 }
 
@@ -2417,7 +2560,7 @@ int Project::takeScheduleManager( ScheduleManager *sm )
             sm->setParentManager( 0 );
             m_managerIdMap.remove( sm->managerId() );
             emit scheduleManagerRemoved( sm );
-            emit changed();
+            emit projectChanged();
         }
     } else {
         index = indexOf( sm );
@@ -2426,7 +2569,7 @@ int Project::takeScheduleManager( ScheduleManager *sm )
             m_managers.removeAt( indexOf( sm ) );
             m_managerIdMap.remove( sm->managerId() );
             emit scheduleManagerRemoved( sm );
-            emit changed();
+            emit projectChanged();
         }
     }
     return index;
@@ -2533,34 +2676,38 @@ void Project::insertCalendarId( const QString &id, Calendar *calendar )
     calendarIdDict.insert( id, calendar );
 }
 
-void Project::changed( Node *node )
+void Project::changed( Node *node, int property )
 {
     if ( m_parent == 0 ) {
-        emit nodeChanged( node );
-        emit changed();
+        Node::changed( node, property ); // reset cache
+        if ( property != Node::Type ) {
+            // add/remove node is handled elsewhere
+            emit nodeChanged( node );
+            emit projectChanged();
+        }
         return;
     }
-    Node::changed( node );
+    Node::changed( node, property );
 }
 
 void Project::changed( ResourceGroup *group )
 {
     //kDebug(planDbg());
     emit resourceGroupChanged( group );
-    emit changed();
+    emit projectChanged();
 }
 
 void Project::changed( ScheduleManager *sm )
 {
     emit scheduleManagerChanged( sm );
-    emit changed();
+    emit projectChanged();
 }
 
 void Project::changed( MainSchedule *sch )
 {
     //kDebug(planDbg())<<sch->id();
     emit scheduleChanged( sch );
-    emit changed();
+    emit projectChanged();
 }
 
 void Project::sendScheduleToBeAdded( const ScheduleManager *sm, int row )
@@ -2572,7 +2719,7 @@ void Project::sendScheduleAdded( const MainSchedule *sch )
 {
     //kDebug(planDbg())<<sch->id();
     emit scheduleAdded( sch );
-    emit changed();
+    emit projectChanged();
 }
 
 void Project::sendScheduleToBeRemoved( const MainSchedule *sch )
@@ -2585,25 +2732,25 @@ void Project::sendScheduleRemoved( const MainSchedule *sch )
 {
     //kDebug(planDbg())<<sch->id();
     emit scheduleRemoved( sch );
-    emit changed();
+    emit projectChanged();
 }
 
 void Project::changed( Resource *resource )
 {
     emit resourceChanged( resource );
-    emit changed();
+    emit projectChanged();
 }
 
 void Project::changed( Calendar *cal )
 {
     emit calendarChanged( cal );
-    emit changed();
+    emit projectChanged();
 }
 
 void Project::changed( StandardWorktime *w )
 {
     emit standardWorktimeChanged( w );
-    emit changed();
+    emit projectChanged();
 }
 
 bool Project::addRelation( Relation *rel, bool check )
@@ -2618,7 +2765,7 @@ bool Project::addRelation( Relation *rel, bool check )
     rel->parent()->addDependChildNode( rel );
     rel->child()->addDependParentNode( rel );
     emit relationAdded( rel );
-    emit changed();
+    emit projectChanged();
     return true;
 }
 
@@ -2628,7 +2775,7 @@ void Project::takeRelation( Relation *rel )
     rel->parent() ->takeDependChildNode( rel );
     rel->child() ->takeDependParentNode( rel );
     emit relationRemoved( rel );
-    emit changed();
+    emit projectChanged();
 }
 
 void Project::setRelationType( Relation *rel, Relation::Type type )
@@ -2636,7 +2783,7 @@ void Project::setRelationType( Relation *rel, Relation::Type type )
     emit relationToBeModified( rel );
     rel->setType( type );
     emit relationModified( rel );
-    emit changed();
+    emit projectChanged();
 }
 
 void Project::setRelationLag( Relation *rel, const Duration &lag )
@@ -2644,7 +2791,7 @@ void Project::setRelationLag( Relation *rel, const Duration &lag )
     emit relationToBeModified( rel );
     rel->setLag( lag );
     emit relationModified( rel );
-    emit changed();
+    emit projectChanged();
 }
 
 QList<Node*> Project::flatNodeList( Node *parent )
