@@ -55,7 +55,7 @@ static double GammaHelp(double& x, bool& reflect)
     double res, anum;
     res = 1.0;
     anum = x;
-    for (int i = 0; i < 6; i++) {
+    for (int i = 0; i < 6; ++i) {
         anum += 1.0;
         res += c[i] / anum;
     }
@@ -760,7 +760,7 @@ Value ValueCalc::fact(int which, int end)
     Value res = Value(1);
     while (which > end) {
         res = mul(res, which);
-        which--;
+        --which;
     }
     return res;
 }
@@ -851,7 +851,7 @@ Value ValueCalc::base(const Value &val, int base, int prec, int minLength)
         result += '.'; value = value - (int)numToDouble(value);
 
         int ix;
-        for (int i = 0; i < prec; i++) {
+        for (int i = 0; i < prec; ++i) {
             ix = (int) numToDouble(value * base);
             result += "0123456789abcdefghijklmnopqrstuvwxyz"[ix];
             value = base * (value - (double)ix / base);
@@ -1063,7 +1063,7 @@ Value ValueCalc::phi(Value x)
 static double taylor_helper(double* pPolynom, uint nMax, double x)
 {
     double nVal = pPolynom[nMax];
-    for (int i = nMax - 1; i >= 0; i--) {
+    for (int i = nMax - 1; i >= 0; --i) {
         nVal = pPolynom[i] + (nVal * x);
     }
     return nVal;
@@ -1369,7 +1369,7 @@ Value ValueCalc::GetLogGamma(Value _x)
 //   double cut1 = dx - dgamma;
 //   double cut2 = dx * 10000000000.0;
 //
-//   for ( int i=1; i<=maxit; i++ ) {
+//   for ( int i=1; i<=maxit; ++i ) {
 //     double ai = i;
 //     term = dx * term / ( dgamma + ai );
 //     sum += term;
@@ -1456,7 +1456,7 @@ Value ValueCalc::GetGammaDist(Value _x, Value _alpha, Value _beta)
         pn3 = x + 1.0;
         pn4 = x * b;
         sum = pn3 / pn4;
-        for (n = 1; ; n++) {
+        for (n = 1; ; ++n) {
 //       kDebug()<<"n="<<n<<" sum="<< sum;
             a += 1.0; // =   n+1 -alpha
             b += 2.0; // = 2(n+1)-alph+x
@@ -1539,7 +1539,7 @@ Value ValueCalc::GetBeta(Value _x, Value _alpha,
             cf = a2 * fnorm;
         }
         cfnew = 1.0;
-        for (uint j = 1; j <= 100; j++) {
+        for (uint j = 1; j <= 100; ++j) {
             rm = (double) j;
             apl2m = fA + 2.0 * rm;
             d2m = rm * (fB - rm) * x / ((apl2m - 1.0) * apl2m);
@@ -1880,8 +1880,8 @@ void ValueCalc::twoArrayWalk(const Value &a1, const Value &a2,
         res = Value::errorVALUE();
         return;
     }
-    for (int r = 0; r < rows; r++)
-        for (int c = 0; c < cols; c++) {
+    for (int r = 0; r < rows; ++r)
+        for (int c = 0; c < cols; ++c) {
             Value v1 = a1.element(c, r);
             Value v2 = a2.element(c, r);
             if (v1.isArray() && v2.isArray())
@@ -1961,8 +1961,8 @@ Value ValueCalc::sumIf(const Value &range, const Condition &cond)
 
     unsigned int rows = range.rows();
     unsigned int cols = range.columns();
-    for (unsigned int r = 0; r < rows; r++)
-        for (unsigned int c = 0; c < cols; c++) {
+    for (unsigned int r = 0; r < rows; ++r)
+        for (unsigned int c = 0; c < cols; ++c) {
             Value v = range.element(c, r);
 
             if (v.isArray())
@@ -1999,17 +1999,15 @@ Value ValueCalc::sumIf(const Cell &sumRangeStart, const Value &range, const Cond
 
     unsigned int rows = range.rows();
     unsigned int cols = range.columns();
-    for (unsigned int r = 0; r < rows; r++)
-        for (unsigned int c = 0; c < cols; c++) {
+    for (unsigned int r = 0; r < rows; ++r)
+        for (unsigned int c = 0; c < cols; ++c) {
             Value v = range.element(c, r);
-
             if (v.isArray())
                 return Value::errorVALUE();
 
             if (matches(cond, v)) {
                 Value val = Cell(sumRangeStart.sheet(), sumRangeStart.column() + c, sumRangeStart.row() + r).value();
                 if (val.isNumber()) {// only add numbers, no conversion from string allowed
-                    //kDebug()<<"add "<<val;
                     res = add(res, val);
                 }
             }
@@ -2018,6 +2016,177 @@ Value ValueCalc::sumIf(const Cell &sumRangeStart, const Value &range, const Cond
     return res;
 }
 
+Value ValueCalc::sumIfs(const Cell &sumRangeStart, QList<Value> range, QList<Condition> cond, const float limit)
+{
+    if(range[0].isError())
+        return range[0];
+
+    Value res(0);
+    Value val;
+
+    unsigned int rows = range[0].rows();
+    unsigned int cols = range[0].columns();
+    for (unsigned int r = 0; r < rows; ++r) {
+        for (unsigned int c = 0; c < cols; ++c) {
+            for (unsigned int i = 1; i <= limit ; ++i) {
+
+                if(range[i].isError())
+                    return range[0];
+
+                if (!range[i].isArray()) {
+                    if (matches(cond[i-1], range[i].element(0, 0))) {
+                        return sumRangeStart.value();
+                    }
+                    return Value(0.0);
+                }
+
+                Value v = range[i].element(c, r);
+                if (v.isArray()) {
+                    return Value::errorVALUE();
+                }
+
+                if (!matches(cond[i-1], v)) {
+                    val = Value(0.0);
+                    break;
+                }
+                val = range[0].element(c, r);
+            }
+            if (val.isNumber()) {// only add numbers, no conversion from string allowed
+                res = add(res, val);
+            }
+        }
+    }
+    return res;
+}
+
+Value ValueCalc::averageIf(const Value &range, const Condition &cond)
+{
+    if(range.isError())
+        return range;
+
+    if (!range.isArray()) {
+        if (matches(cond, range.element(0, 0))) {
+            return range;
+        }
+        return Value(0.0);
+    }
+
+    Value res(0);
+    Value tmp;
+
+    unsigned int rows = range.rows();
+    unsigned int cols = range.columns();
+    unsigned int cnt = 0;
+    for (unsigned int r = 0; r < rows; ++r)
+        for (unsigned int c = 0; c < cols; ++c) {
+            Value v = range.element(c, r);
+
+            if (v.isArray())
+                tmp = averageIf(v, cond);
+            if (tmp.isNumber()) {// only add numbers, no conversion from string allowed
+                res = add(res, tmp);
+            } else if (matches(cond, v)) {
+                if (v.isNumber()) {// only add numbers, no conversion from string allowed
+                    //kDebug()<<"add "<<v;
+                    res = add(res, v);
+                    ++cnt;
+                }
+            }
+        }
+
+    res = div(res, cnt);
+    return res;
+}
+
+Value ValueCalc::averageIf(const Cell &avgRangeStart, const Value &range, const Condition &cond)
+{
+    if(range.isError())
+        return range;
+
+    if (!range.isArray()) {
+        if (matches(cond, range.element(0, 0))) {
+            return avgRangeStart.value();
+        }
+        return Value(0.0);
+    }
+
+    Value res(0);
+
+    unsigned int rows = range.rows();
+    unsigned int cols = range.columns();
+    unsigned int cnt = 0;
+    for (unsigned int r = 0; r < rows; ++r)
+        for (unsigned int c = 0; c < cols; ++c) {
+            Value v = range.element(c, r);
+
+            if (v.isArray())
+                return Value::errorVALUE();
+
+            if (matches(cond, v)) {
+                Value val = Cell(avgRangeStart.sheet(), avgRangeStart.column() + c, avgRangeStart.row() + r).value();
+                if (val.isNumber()) {// only add numbers, no conversion from string allowed
+                    //kDebug()<<"add "<<val;
+                    res = add(res, val);
+                    ++cnt;
+                }
+            }
+        }
+
+    res = div(res, cnt);
+    return res;
+}
+
+Value ValueCalc::averageIfs(const Cell &avgRangeStart, QList<Value> range, QList<Condition> cond, const float limit)
+{
+    if(range[0].isError())
+        return range[0];
+
+    Value res(0);
+    Value val;
+
+    unsigned int rows = range[0].rows();
+    unsigned int cols = range[0].columns();
+    unsigned int cnt = 0;
+    for (unsigned int r = 0; r < rows; ++r) {
+        for (unsigned int c = 0; c < cols; ++c) {
+            bool flag = true;
+            for (unsigned int i = 1; i <= limit ; ++i) {
+
+                if(range[i].isError())
+                    return range[0];
+
+                if (!range[i].isArray()) {
+                    if (matches(cond[i-1], range[i].element(0, 0))) {
+                        return avgRangeStart.value();
+                    }
+                    return Value(0.0);
+                }
+
+                Value v = range[i].element(c, r);
+                if (v.isArray()) {
+                    return Value::errorVALUE();
+                }
+
+                if (!matches(cond[i-1], v)) {
+                    flag = false;
+                    val = Value(0.0);
+                    break;
+                }
+                val = range[0].element(c, r);
+            }
+            if (flag) {
+                ++cnt;
+                flag = true;
+            }
+            if (val.isNumber()) {// only add numbers, no conversion from string allowed
+                res = add(res, val);
+            }
+        }
+    }
+    res = div(res, cnt);
+    return res;
+}
+
 int ValueCalc::count(const Value &range, bool full)
 {
     Value res(0);
@@ -2049,12 +2218,58 @@ int ValueCalc::countIf(const Value &range, const Condition &cond)
         if (v.isArray())
             res += countIf(v, cond);
         else if (matches(cond, v))
-            res++;
+            ++res;
     }
 
     return res;
 }
 
+Value ValueCalc::countIfs(const Cell &cntRangeStart, QList<Value> range, QList<Condition> cond, const float limit)
+{
+    if (!range[0].isArray())
+        return Value(0.0);
+
+    if(range[0].isError())
+        return range[0];
+
+    Value res(0);
+
+    unsigned int rows = range[0].rows();
+    unsigned int cols = range[0].columns();
+    for (unsigned int r = 0; r < rows; ++r) {
+        for (unsigned int c = 0; c < cols; ++c) {
+            bool flag = true;
+            for (unsigned int i = 0; i <= limit ; ++i) {
+
+                if(range[i].isError())
+                    return range[0];
+
+                if (!range[i].isArray()) {
+                    if (matches(cond[i], range[i].element(0, 0))) {
+                        return cntRangeStart.value();
+                    }
+                    return Value(0.0);
+                }
+
+                Value v = range[i].element(c, r);
+                if (v.isArray()) {
+                    return Value::errorVALUE();
+                }
+
+                if (!matches(cond[i], v)) {
+                    flag = false;
+                    break;
+                }
+            }
+            if (flag) {
+                res = add(res, 1);
+                flag = true;
+            }
+        }
+    }
+    return res;
+}
+
 Value ValueCalc::avg(const Value &range, bool full)
 {
     int cnt = count(range, full);
@@ -2202,7 +2417,6 @@ Value::Format ValueCalc::format(Value a, Value b)
     return af;
 }
 
-
 // ------------------------------------------------------
 
 void ValueCalc::getCond(Condition &cond, Value val)
@@ -2219,25 +2433,25 @@ void ValueCalc::getCond(Condition &cond, Value val)
     cond.comp = isEqual;
     text = text.trimmed();
 
-    if (text.startsWith("<=")) {
+    if (text.startsWith(QLatin1String("<="))) {
         cond.comp = lessEqual;
         text = text.remove(0, 2);
-    } else if (text.startsWith(">=")) {
+    } else if (text.startsWith(QLatin1String(">="))) {
         cond.comp = greaterEqual;
         text = text.remove(0, 2);
-    } else if (text.startsWith("!=") || text.startsWith("<>")) {
+    } else if (text.startsWith(QLatin1String("!=")) || text.startsWith(QLatin1String("<>"))) {
         cond.comp = notEqual;
         text = text.remove(0, 2);
-    } else if (text.startsWith("==")) {
+    } else if (text.startsWith(QLatin1String("=="))) {
         cond.comp = isEqual;
         text = text.remove(0, 2);
-    } else if (text.startsWith('<')) {
+    } else if (text.startsWith(QLatin1Char('<'))) {
         cond.comp = isLess;
         text = text.remove(0, 1);
-    } else if (text.startsWith('>')) {
+    } else if (text.startsWith(QLatin1Char('>'))) {
         cond.comp = isGreater;
         text = text.remove(0, 1);
-    } else if (text.startsWith('=')) {
+    } else if (text.startsWith(QLatin1Char('='))) {
         cond.comp = isEqual;
         text = text.remove(0, 1);
     }
