@@ -75,7 +75,7 @@
 #include <kparts/event.h>
 #include <kpushbutton.h>
 #include <kxmlguifactory.h>
-#include <knotifyconfigwidget.h>
+#include <kservicetypetrader.h>
 
 // Calligra includes
 #include <KoGlobal.h>
@@ -160,6 +160,7 @@
 // D-Bus
 #ifndef QT_NO_DBUS
 #include "interfaces/ViewAdaptor.h"
+#include <knotifyconfigwidget.h>
 #include <QtDBus>
 #endif
 
@@ -1584,8 +1585,10 @@ void View::showTabBar(bool enable)
 }
 
 void View::optionsNotifications()
-{
+{  
+#ifndef QT_NO_DBUS
     KNotifyConfigWidget::configure(this);
+#endif
 }
 
 void View::preference()
