@@ -20,6 +20,7 @@
 #ifndef KPRPRESENTATIONTOOLADAPTOR_H
 #define KPRPRESENTATIONTOOLADAPTOR_H
 
+#ifndef QT_NO_DBUS
 #include "KPrPresentationTool.h"
 
 #include <QtDBus>
@@ -74,5 +75,6 @@ private:
     KPrPresentationTool *m_tool;
     KPrViewModePresentation &m_viewModePresentation;
 };
+#endif // QT_NO_DBUS
 
 #endif /* KPRPRESENTATIONTOOLADAPTOR_H */
