@@ -46,9 +46,11 @@
 #include <KoStoreDevice.h>
 #include <KoDocumentRdfBase.h>
 
+#include "author/CoverImage.h"
+
 #include <QBuffer>
 #include <QTextCursor>
-#include <KDebug>
+#include <kdebug.h>
 #include <ktemporaryfile.h>
 
 static const struct {
@@ -87,7 +89,7 @@ QByteArray KWOdfWriter::serializeHeaderFooter(KoShapeSavingContext &context, KWT
     Q_ASSERT(shapedata);
 
     writer.startElement(tag);
-    shapedata->saveOdf(context, m_document->documentRdfBase());
+    shapedata->saveOdf(context, m_document->documentRdf());
     writer.endElement();
 
     context.setOptions(options);
@@ -118,7 +120,7 @@ void KWOdfWriter::saveHeaderFooter(KoShapeSavingContext &context)
     }
 
     // save page styles that don't have a header or footer which will be handled later
-    foreach (KWPageStyle pageStyle, m_document->pageManager()->pageStyles()) {
+    foreach (const KWPageStyle &pageStyle, m_document->pageManager()->pageStyles()) {
         if (data.contains(pageStyle))
             continue;
 
@@ -138,7 +140,7 @@ void KWOdfWriter::saveHeaderFooter(KoShapeSavingContext &context)
           << Words::OddPagesFooterTextFrameSet
           << Words::EvenPagesFooterTextFrameSet;
 
-    foreach (KWPageStyle pageStyle, data.keys()) {
+    foreach (const KWPageStyle &pageStyle, data.keys()) {
         KoGenStyle masterStyle(KoGenStyle::MasterPageStyle);
         //masterStyle.setAutoStyleInStylesDotXml(true);
         KoGenStyle layoutStyle = pageStyle.saveOdf();
@@ -223,6 +225,8 @@ bool KWOdfWriter::save(KoOdfWriteStore &odfStore, KoEmbeddedDocumentSaver &embed
 
     KoGenChanges changes;
 
+    CoverImage coverImage;
+
     KoChangeTracker *changeTracker = m_document->resourceManager()->resource(KoText::ChangeTracker).value<KoChangeTracker*>();
 
     KoShapeSavingContext context(*tmpBodyWriter, mainStyles, embeddedSaver);
@@ -246,11 +250,10 @@ bool KWOdfWriter::save(KoOdfWriteStore &odfStore, KoEmbeddedDocumentSaver &embed
     KoXmlWriter *bodyWriter = odfStore.bodyWriter();
     bodyWriter->startElement("office:body");
     bodyWriter->startElement("office:text");
-
-    // Save user defined variable declarations
-    if (KoVariableManager *variableManager = m_document->inlineTextObjectManager()->variableManager()) {
-        variableManager->saveOdf(bodyWriter);
+    if (m_document->isMasterDocument()) {
+        bodyWriter->addAttribute("text:global", "true");
     }
+    // FIXME: text:use-soft-page-breaks
 
     calculateZindexOffsets();
 
@@ -265,7 +268,8 @@ bool KWOdfWriter::save(KoOdfWriteStore &odfStore, KoEmbeddedDocumentSaver &embed
 
         if (fs->frameCount() == 1) {
             // may be a frame that is anchored to text, don't save those here.
-            if (fs->frames().first()->anchorType() != KoTextAnchor::AnchorPage)
+            KoShapeAnchor *anchor = fs->frames().first()->shape()->anchor();
+            if (anchor && anchor->anchorType() != KoShapeAnchor::AnchorPage)
                 continue;
         }
 
@@ -302,15 +306,7 @@ bool KWOdfWriter::save(KoOdfWriteStore &odfStore, KoEmbeddedDocumentSaver &embed
         if (! mainTextFrame->frames().isEmpty() && mainTextFrame->frames().first()) {
             KoTextShapeData *shapeData = qobject_cast<KoTextShapeData *>(mainTextFrame->frames().first()->shape()->userData());
             if (shapeData) {
-                KWPageManager *pm = m_document->pageManager();
-                if (pm->pageCount()) { // make the first page refer to our page master
-                    QTextCursor cursor(shapeData->document());
-                    QTextBlockFormat tbf;
-                    KWPageStyle style = pm->pages().first().pageStyle();
-                    tbf.setProperty(KoParagraphStyle::MasterPageName, m_masterPages.value(style));
-                    cursor.mergeBlockFormat(tbf);
-                }
-                shapeData->saveOdf(context, m_document->documentRdfBase());
+                shapeData->saveOdf(context, m_document->documentRdf());
             }
         }
     }
@@ -334,12 +330,17 @@ bool KWOdfWriter::save(KoOdfWriteStore &odfStore, KoEmbeddedDocumentSaver &embed
 
     bodyWriter->addCompleteElement(&tmpChangeFile);
 
+    // Save user defined variable declarations
+    if (KoVariableManager *variableManager = m_document->inlineTextObjectManager()->variableManager()) {
+        variableManager->saveOdf(bodyWriter);
+    }
+
     // Do not write out text:page-sequence, if there is a maintTextFrame
     // The ODF specification does not allow text:page-sequence in office:text
     // if there is e.g. text:p or text:h there
     if (!mainTextFrame) {
         bodyWriter->startElement("text:page-sequence");
-        foreach (KWPage page, m_document->pageManager()->pages()) {
+        foreach (const KWPage &page, m_document->pageManager()->pages()) {
             Q_ASSERT(m_masterPages.contains(page.pageStyle()));
             bodyWriter->startElement("text:page");
             bodyWriter->addAttribute("text:master-page-name",
@@ -365,7 +366,7 @@ bool KWOdfWriter::save(KoOdfWriteStore &odfStore, KoEmbeddedDocumentSaver &embed
 
     // update references to xml:id to be to new xml:id
     // in the external Rdf
-    if (KoDocumentRdfBase *rdf = m_document->documentRdfBase()) {
+    if (KoDocumentRdfBase *rdf = m_document->documentRdf()) {
         QMap<QString, QString> m = sharedData->getRdfIdMapping();
         rdf->updateXmlIdReferences(m);
     }
@@ -377,6 +378,10 @@ bool KWOdfWriter::save(KoOdfWriteStore &odfStore, KoEmbeddedDocumentSaver &embed
         return false;
     }
 
+    // save cover image in Author.
+    if (!coverImage.saveCoverImage(store, manifestWriter, m_document->coverImage())) {
+        return false;
+    }
     return true;
 }
 
